/* $Id: DVMtest.c,v 1.4 91/02/06 11:52:58 cwcork Exp Locker: cwcork $
 *
 * $File: DVMtest.c $
 *
 * $Revision: 1.4 $
 * $State: Exp $
 * $Author: cwcork $
 *
 * $Log:	DVMtest.c,v $
 * Revision 1.4  91/02/06  11:52:58  cwcork
 * remove $ from defined constants.
 * 
 * Revision 1.3  91/02/06  11:38:59  cwcork
 * Changes to use ../include files.
 * 
 * Revision 1.2  90/09/17  14:04:48  cwcork
 * Add error handling to Mv300Dev routines.
 * 
 * Revision 1.1  90/08/03  15:01:17  cwcork
 * Initial revision
 * 
 *
 * PURPOSE:
 *   test of HP 3458A DVM
 *
 * COMMENTS:
 * KNOWN BUGS:
 */
#include "vxWorks.h"
#include "ioLib.h"
#include <stdio.h>
#include "Mv300Dev.h"
#include "Mv300Brd.h"

DVMtest ()
{
  MV300_BRD_ID	brdId;
  MV300_DEV_ID  devId;
  MV300_CONFIG  devConfig;
  MV300_STATUS  brdStatus;
  STATUS	status;
  INT		options;
  BYTE		cmdBuf[16];
  BYTE 		bufOut[128];
  BYTE		bufIn[128];
  UINT		bufOutLen = sizeof (bufOut);
  UINT		bufInLen = sizeof (bufIn) - 1;
  UINT		bytesTransferred = 0;
  CHAR		command;
  BOOL		quit = FALSE;

  printf ("\n------HP3458A DVM test------\n\n");

  /* set up ios structure */
  devConfig.timeout = 300;	/* 5 second timeout */
  devConfig.flags = (MV300_CONFIG_V_REOS |
		     MV300_CONFIG_V_WEOS |
                     MV300_CONFIG_V_DMA  |
		     MV300_CONFIG_V_EOT);
  devConfig.priAddr = 22;	        /* primary address */
  devConfig.secAddr = 0;	        /* no secondary addressing */
  devConfig.reosChar = '\r';
  devConfig.weosChar = '\r';

  if ((status = Mv300BrdCreate (&brdId,
				"esg2", 0xffff0100, 200, 3, &devConfig)) != OK)
    {
      printf ("DVMtest: Mv300BrdCreate Failure, status = %d\n", status);
      return;
    }
  else
    {
      printf ("DVMtest: Mv300BrdCreate OK, brdId = 0x%x\n", brdId);
    }

  /* inialize device structure */
  if ((status = Mv300DevCreate (&devId, brdId, &devConfig)) != OK)
    {
      printf ("DVMtest: Mv300DevCreate Failure, status = %d\n", status);
      return;
    }
  else
    {
      printf ("DVMtest: Mv300DevCreate OK, brdId = 0x%x\n", brdId);
    }

  for (;;)
    {
      printf ("\nCommand?: ");
      gets (cmdBuf);
      sscanf (cmdBuf, "%c", &command);

      switch (command)
	{
	case 'w':
	case 'W':
	  bufOutLen = strlen (&cmdBuf[2]);
	  if ((status = Mv300DevWrite (devId,
				       &cmdBuf[2],
				       bufOutLen,
				       &bytesTransferred)) == OK)
	    {
	      printf ("Wrote: %s", &cmdBuf[2]);
	    }
	  else
	    {
	      printf ("Write error: status = %d\n", status);
	    }
	  break;
	case 'r':
	case 'R':
	  if ((status = Mv300DevRead (devId,
				      bufIn,
				      bufInLen,
				      &bytesTransferred)) == OK)
	    {
	      bufIn[bytesTransferred] = '\0';
	      printf ("Read: %s\n", bufIn);
	    }
	  else
	    {
	      printf ("Read error: status = %d\n", status);
	    }
	  break;
	case 'Q':
	case 'q':
	  printf ("QUIT\n");
	  quit = TRUE;
	  break;
	default:
	  printf ("Unrecognized command: %c, 0x%x\n", command, command);
	  break;
	}
      if (quit)
	break;
    }

  /* Free up resources */
  printf ("DVMtest: Mv300DevForget\n");
  if ((status = Mv300DevForget (devId)) == OK)
    {
      printf ("DVMtest: Mv300DevForget STATUS =   OK (0)\n");
    }
  else
    {

      printf ("DVMtest: Mv300DevForget STATUS = FAIL (%d)\n", status);
    }

  printf ("DVMtest: Mv300BrdForget\n");
  if ((status = Mv300BrdForget (brdId)) == OK)
    {
      printf ("DVMtest: Mv300BrdForget STATUS =   OK (0)\n");
    }
  else
    {

      printf ("DVMtest: Mv300BrdForget STATUS = FAIL (%d)\n", status);
    }

  return;
}
