/* $Id: Mv300.h,v 1.5 91/02/06 11:38:20 cwcork Exp Locker: cwcork $
 *
 * $File: mv300.h $
 *
 * $Revision: 1.5 $
 * $State: Exp $
 * $Author: cwcork $
 *
 * (C) Copyright 1990 Carl Cork. All rights reserved.
 * Permission to copy for any purpose is hereby granted so long
 * as this copyright notice remains intact.
 *
 * $Log:	Mv300.h,v $
 * Revision 1.5  91/02/06  11:38:20  cwcork
 * Changes to use ../include files.
 * 
 * Revision 1.4  91/01/26  08:58:19  cwcork
 * Move most error codes into ../base/Base.h.
 * replace $ with _ in all #define declarations (for ANSI C compatibility).
 * 
 * Revision 1.3  90/09/10  14:12:09  cwcork
 * Add MV300$_ERMT error code for remote procedure calls.
 * 
 * Revision 1.2  90/07/27  17:57:06  cwcork
 * Revise status bit arrangement.
 * Add include for vwModNum.h
 * 
 * Revision 1.1  90/07/10  16:20:49  cwcork
 * Initial revision
 * 
 *
 * PURPOSE:
 *  Descriptor File,
 *  for Motorola MVME300 IEEE-488 (GPIB) bus controller.
 *
 * COMMENTS:
 * KNOWN BUGS:
 */
#ifndef MV300_H
#define MV300_H 1

/*======================== External Declarations =============================*/
#include "Base.h"
#include "vwModNum.h"

/* Error codes specific to Mv300 modules */
#define ERROR_ECIC		101  /* Must be CIC to use this function */
#define ERROR_ENOL		102  /* No listener for write function */
#define ERROR_ESAC		103  /* Must be SC to use this function */
#define ERROR_EBID		104  /* Invalid board structure ID */
#define ERROR_EDID		105  /* Invalid device structure ID */
#define ERROR_EBRD		106 /* Invalid board address */

/* IOCtl function codes */

    /* DEVICE CONTROL FUNCTIONS */
#define MV300_K_DCLEAR	    (M_mv300Lib | 0x92)  /* Clear a device */
#define MV300_K_DLOCAL	    (M_mv300Lib | 0x93)  /* Enable local control */
#define MV300_K_PASSCONTROL (M_mv300Lib | 0x96)  /* Pass control */
#define MV300_K_PPOLL	    (M_mv300Lib | 0x97)  /* Parallel poll device */
#define MV300_K_PPOLLCONF   (M_mv300Lib | 0x98)  /* Configure for PPOLL */
#define MV300_K_TRIGGER	    (M_mv300Lib | 0x99)  /* Trigger a device */
#define MV300_K_SPOLL	    (M_mv300Lib | 0x9a)  /* Serial poll device */
#define MV300_K_DCONF_RD    (M_mv300Lib | 0xa1)  /* Return device config */
#define MV300_K_DCONF_WRT   (M_mv300Lib | 0xa2)  /* Update device config */
    /* BOARD CONTROL FUNCTIONS */
#define MV300_K_CMD	    (M_mv300Lib | 0x67)  /* Send GPIB commands */
#define MV300_K_ATNON	    (M_mv300Lib | 0x68)  /* Raise ATN */
#define MV300_K_ATNOFF	    (M_mv300Lib | 0x69)  /* Lower ATN */
#define MV300_K_SETREMOTE   (M_mv300Lib | 0x6b)  /* Set remote enable */
#define MV300_K_SENDIFC	    (M_mv300Lib | 0x6c)  /* Send interface clear */
#define MV300_K_PRESENT	    (M_mv300Lib | 0x6d)  /* Rtn TRUE if board present */
#define MV300_K_ONLINE	    (M_mv300Lib | 0x6e)	/* Place board online */
#define MV300_K_OFFLINE	    (M_mv300Lib | 0x6f)	/* Place board offline */
#define MV300_K_BLOCAL	    (M_mv300Lib | 0x71)	/* Rtn board to local state */
#define MV300_K_SPOLLBYTE   (M_mv300Lib | 0x72)	/* Change serial poll byte */
#define MV300_K_REQUEST	    (M_mv300Lib | 0x73)	/* Set request service */
#define MV300_K_PPOLLRESP   (M_mv300Lib | 0x74)	/* Set parallel poll status */
#define MV300_K_CLRREMOTE   (M_mv300Lib | 0x75)	/* Clear remote enable */
    /* DEVICE AND BOARD CONTROL FUNCTIONS */
#define MV300_K_STATUS	    (M_mv300Lib | 0x70)	/* Return board status */
#define MV300_K_BWAIT	    (M_mv300Lib | 0x76)	/* Wait for event */
#define MV300_K_BCONF_RD    (M_mv300Lib | 0x79)	/* Return board configuration */
#define MV300_K_BCONF_WRT   (M_mv300Lib | 0x80)	/* Update board configuration */

/* MV300_CONFIG - device (and board) configuration structure */

typedef struct		/* MV300_BRD_IOS - MV300_BRD device IO descriptor */
{
  UINT  	timeout;	/* device timeout */
  UINT_2	flags;		/* device configuration flags */
  BYTE		priAddr;	/* device primary address */
  BYTE		secAddr;	/* device secondary address */
  BYTE		reosChar;	/* read EOS character */
  BYTE		weosChar;	/* write EOS character */
} MV300_CONFIG;
typedef MV300_CONFIG* PMV300_CONFIG;

#define MV300_CONFIG_M_FLAGS	0x000f	/* flag mask */
#define MV300_CONFIG_V_EOT	0x0008	/* enable EOI on write */
#define MV300_CONFIG_V_REOS	0x0004	/* terminate read on EOS */
#define MV300_CONFIG_V_WEOS	0x0002	/* send EOS on write (with EOI) */
#define MV300_CONFIG_V_DMA	0x0001	/* do DMA I/O transfers */

/* MV300_STATUS - board status word */

typedef UINT_4 MV300_STATUS;
typedef MV300_STATUS* PMV300_STATUS;

#define MV300_STATUS_V_RES	0x80000000	/* reserved */
#define MV300_STATUS_V_TIMEO	0x40000000	/* timeout */
#define MV300_STATUS_V_BERR     0x20000000	/* bus error */
#define MV300_STATUS_V_SYSFAIL  0x10000000	/* SYSFAIL LED is lit */
#define MV300_STATUS_V_BCRDN    0x08000000	/* byte count register done */
#define MV300_STATUS_V_FULL     0x04000000      /* FIFO full */
#define MV300_STATUS_V_EODNE    0x02000000      /* EOS/EOI received */
#define MV300_STATUS_V_EMPTY    0x01000000      /* FIFO empty */
#define MV300_STATUS_V_SC	0x00800000	/* System Controller */
#define MV300_STATUS_V_CIC	0x00400000	/* Controller In Charge */
#define MV300_STATUS_V_BI       0x00200000	/* Byte In */
#define MV300_STATUS_V_BO       0x00100000	/* Byte out */
#define MV300_STATUS_V_END      0x00080000	/* EOI received */
#define MV300_STATUS_V_SPAS     0x00040000	/* Serial poll received */
#define MV300_STATUS_V_RLC      0x00020000	/* Remote/Local change */
#define MV300_STATUS_V_MAC      0x00010000	/* My address change */
#define MV300_STATUS_V_GET      0x00008000	/* Group execute trigger */
#define MV300_STATUS_V_ERR      0x00004000	/* Error */
#define MV300_STATUS_V_UNC      0x00002000	/* Command error */
#define MV300_STATUS_V_APT      0x00001000	/* Address Pass Through */
#define MV300_STATUS_V_DCAS     0x00000800	/* Device clear active state */
#define MV300_STATUS_V_MA       0x00000400	/* My address */
#define MV300_STATUS_V_SRQ      0x00000200	/* Service request */
#define MV300_STATUS_V_IFC      0x00000100	/* Interface clear */
#define MV300_STATUS_V_REM      0x00000080	/* Remote state */
#define MV300_STATUS_V_ATN      0x00000040	/* Attention */
#define MV300_STATUS_V_LLO      0x00000020	/* Local lockout */
#define MV300_STATUS_V_LPAS     0x00000010	/* Listen Pri addressed state */
#define MV300_STATUS_V_TPAS     0x00000008	/* Talk pri addressed state */
#define MV300_STATUS_V_LADS     0x00000004	/* addressed to listen */
#define MV300_STATUS_V_TADS     0x00000002	/* addressed to talk */
#define MV300_STATUS_V_ULPA     0x00000001	/* LSB last address */

/*======================== Exported Function Prototypes ======================*/
/*========================= Exported Variables ===============================*/
/*================================= END ======================================*/
#endif MV300_H
