/* $Id: Mv300Brd.c,v 1.17 91/02/06 11:39:05 cwcork Exp Locker: cwcork $
 *
 * $File: Mv300Brd.c$
 *
 * $Revision: 1.17 $
 *
 * $State: Exp $
 *
 * $Author: cwcork $
 *
 * (C) Copyright 1990 Carl Cork. All rights reserved.
 * Permission to copy for any purpose is hereby granted so long
 * as this copyright notice remains intact.
 *
 * $Log:	Mv300Brd.c,v $
 * Revision 1.17  91/02/06  11:39:05  cwcork
 * Changes to use ../include files.
 * 
 * Revision 1.16  91/01/28  15:56:55  cwcork
 * Mv300BrdDevRead: get last byte from input register on DMA reads.
 * 
 * Revision 1.15  91/01/26  09:01:48  cwcork
 * Replace $ with _ in all defined constants.
 * Mv300BrdRead : read last byte from input register at end of DMA read.
 * 
 * Revision 1.14  90/09/21  16:00:06  cwcork
 * Mv300BrdRead & Mv300BrdDevRead: unmask IM0_END interrupt before DMAW,
 * else board will not interrupt on EOI.
 * 
 * Revision 1.13  90/09/06  11:54:29  cwcork
 * Mv300BrdCreate: add argument for hostname - used only by RPC interface.
 * 
 * Revision 1.12  90/08/29  10:18:33  cwcork
 * Move MV300_BRD struct from Mv300Brd.h to this module.
 * MV300_BRD_ID now typedef int, but is used internally as PMV300_BRD.
 * This is done to make network RPCs cleaner.
 * 
 * Revision 1.11  90/08/03  15:03:18  cwcork
 * busInit: remove brdWait call to avoid infinite recursion
 *          and to avoid possible lockout in timeout situations.
 * 
 * Revision 1.10  90/07/31  11:14:34  cwcork
 * Debug statements removed. Working Version - Beta release.
 * 
 * Revision 1.9  90/07/31  07:58:28  cwcork
 * Balance int0_mask settings.
 * Clean up bugs. Now works both DMA and PIO.
 * printfs not yet removed.
 * 
 * Revision 1.8  90/07/27  17:59:04  cwcork
 * Check in of buggy code revisions prior to final cleanup.
 * Code changes to reflect added state element for MV300_BRD.
 * Code changes for modified MV300_STATUS bits.
 * Add Mv300BrdRead and Mv300BrdWrite for direct board read/write.
 * mv300BrdInit -> busInit.
 * Add Mv300BrdDevCommand.
 * Mv300BrdCommand: use board config parameters for timeout period.
 * mv300BrdInt -> brdInt: no longer automatically mask interrupts.
 * all functions release bus prior to exit (ie goto standby).
 * general streamlining of mask setting and status handling.
 * 
 * Revision 1.7  90/07/10  16:22:34  cwcork
 * Add numerous new functions.
 * Fix DMA feature, but still leave debug statements in.
 * Ready for cleanup.
 * 
 * Revision 1.6  90/06/21  16:24:03  cwcork
 * Add new function Mv300BrdAddr.
 * Rename Mv300BrdRd -> Mv300BrdRead & revise arguments.
 * Rename Mv300BrdWrt -> Mv300BrdWrite & revise arguments.
 * Revise Mv300BrdCreate: change return value to type STATUS and
 *   modify argument structure.
 * Add feature to disable watchdog timeout.
 * 
 * Revision 1.5  90/06/19  13:06:06  cwcork
 * Programmed I/O now working.
 * Robust error recovery not yet implemented.
 * 
 * Revision 1.4  90/06/15  17:10:16  cwcork
 * Temporary snapshot of cleaned up routines. DMA still buggy.
 * Syntax error during write still needs looking into.
 * 
 * Revision 1.3  90/06/15  16:13:56  cwcork
 * Temporary snapshot of development, dma still broken.
 * 
 * Revision 1.2  90/06/06  07:33:09  cwcork
 * First implementation with interrupts.
 * 
 * Revision 1.1  90/06/04  13:21:45  cwcork
 * Initial revision
 * 
 *
 * PURPOSE:
 *   This is a board control module for
 *   the Motorola MVME300 IEEE-488 bus controller.
 *
 * FUNCTIONS:
 *  Exported-
 *   --Create and Forget--
 *   Mv300BrdCreate	- Create and initialize the board and control structure.
 *   Mv300BrdForget	- Make board offline and free control structure memory.
 *   --Board operations--
 *   Mv300BrdAtn	- Raise/Lower ATN
 *   Mv300BrdCommand	- Send GPIB multiline commands.
 *   Mv300BrdConfRd	- Read board configuration.
 *   Mv300BrdConfWrt	- Write board configuration.
 *   Mv300BrdLocal	- Return board to local state.
 *   Mv300BrdOffline	- Make board offline and disable interrupts.
 *   Mv300BrdOnline	- Make board online and enable interrupts.
 *   Mv300BrdPPoll	- Parallel poll the bus.
 *   Mv300BrdPPollByte	- Set TMS Parallel Poll register.
 *   Mv300BrdRead	- Read from board data_in register.
 *   Mv300BrdRemote	- Set/Clear remote enable
 *   Mv300BrdRequest	- Set request service.
 *   Mv300BrdSendIfc	- Send IFC command
 *   Mv300BrdSPollByte	- Set TMS Serial Poll register.
 *   Mv300BrdStatus	- Get current board status.
 *   Mv300BrdWrite	- Write to board data_out register.
 *   --Device Operations--
 *   Mv300BrdDevCommand - Send GPIB multiline commands using device config.
 *   Mv300BrdDevRead	- Read device data.
 *   Mv300BrdDevSPoll	- Serial poll a device
 *   Mv300BrdDevWrite	- Write device data.
 *  Local-
 *   busInit		- Initialize bus interface hardware.
 *   brdInt		- Exported interrupt handler.
 *   brdOffline		- Place board offline.
 *   brdOnline		- Place board online.
 *   brdTimeout		- Exported watchdog timeout handler.
 *   brdWait		- Wait for return from interrupt and process errors.
 *
 * COMMENTS:
 *
 * KNOWN BUGS:
 */
/*========================= External Declarations ============================*/
#define  MV300BRD_C

#include "vxWorks.h"
#include "iv68k.h"
#include "memLib.h"
#include "semLib.h"
#include "vme.h"
#include "Mv300Brd.h"
/*========================= Local Declarations ===============================*/
/* Initialization constants */
#define REL_CNT		1	/* bus release count */

/* MV300_BRD state bit definitions */
#define BRD_TIMEOUT	0x04	/* timeout */
#define BRD_SC		0x02	/* system controller */
#define BRD_CIC		0x01	/* Controller-In-Charge */

/* MV300_BRD_STATUS - board status */
/* NOTE: this will be equivalenced to MV300_STATUS */
typedef  struct
{
  BYTE  brd_stat;	/* csreg plus timeout and done flags */
  BYTE  int0_stat;	/* TMS9914 int0 status register */
  BYTE  int1_stat;	/* TMS9914 int1 status register */
  BYTE  addr_stat;	/* TMS9914 addr status register */
} MV300_BRD_STATUS;

/* MV300_BRD: board control structure */
typedef struct 		/* MV300_BRD - MV300 controller structure */
{
  ADDR         		port;		/* MV300 i/o port base address */
  WDOG_ID		wDogId;		/* watchdog timer */
  SEM_ID		brdSemId;	/* board mutex id */
  SEM_ID		intSemId;	/* interrupt semaphore id */
  INT			intNumber;	/* interrupt vector number */
  INT			intLevel; 	/* interrupt level */
  INT_4			state;		/* board state bits */
  MV300_CONFIG		config;		/* board configuration */
  MV300_BRD_STATUS      status;		/* board status */
} MV300_BRD;

typedef MV300_BRD* MV300_BRD_ID;

/* MV300_PORT: MVME300 IO registers */
typedef struct
{
  BYTE   fill00;
  BYTE   mar0;	    /* memory DMA address, LSByte */
  BYTE   fill02;
  BYTE   mar1;	    /* memory DMA address, Middle Byte */
  BYTE   fill04;
  BYTE   mar2;	    /* memory DMA address, MSByte */
  BYTE   fill06;
  BYTE   bcr0;	    /* DMA count, LSByte */
  BYTE   fill08;
  BYTE   bcr1;	    /* DMA count, MSByte */
  BYTE   fill0a;
  BYTE   gcrst;	    /* Reset */
  BYTE   fill0c;
  BYTE   amr;	    /* Address Modifier */
  BYTE   fill0e;
  BYTE   csreg;	    /* Control Status register */
  BYTE   fill10;
  BYTE   vbrcr;	    /* Release count register */
  BYTE   fill12;
  BYTE   eoscr;	    /* EOS */
  BYTE   fill14;
  BYTE   ucr;	    /* Uniline */
  BYTE   fill16;
  BYTE   fdior;	    /* FIFO */
  BYTE   unused[8];
  union 	    /* TMS9914A */
    {
      struct
	{
	  BYTE   fill20;
          BYTE   int0_stat;
          BYTE   fill22;
          BYTE   int1_stat;
          BYTE   fill24;
          BYTE   addr_stat;
          BYTE   fill26;
          BYTE   bus_stat;
          BYTE   fill28;
          BYTE   addr_sw1;
          BYTE   fill2a;
          BYTE   fill2b;
          BYTE   fill2c;
          BYTE   cmd_pass;
          BYTE   fill2e;
          BYTE   data_in;
        } r;
      struct
        {
          BYTE   fill20;
          BYTE   int0_mask;
          BYTE   fill22;
          BYTE   int1_mask;
          BYTE   fill24;
          BYTE   unused;
          BYTE   fill26;
          BYTE   aux_cmd;
          BYTE   fill28;
          BYTE   addr_reg;
          BYTE   fill2a;
          BYTE   ser_poll;
          BYTE   fill2c;
          BYTE   par_poll;
          BYTE   fill2e;
          BYTE   data_out;
        } w;
      } tms;
  BYTE   fill30;
  BYTE   cr0;		    /* BIM MC68153 */
  BYTE   fill32;
  BYTE   cr1;
  BYTE   fill34;
  BYTE   cr2;
  BYTE   fill36;
  BYTE   cr3;
  BYTE   fill38;
  BYTE   vr0;
  BYTE   fill3a;
  BYTE   vr1;
  BYTE   fill3c;
  BYTE   vr2;
  BYTE   fill3e;
  BYTE   vr3;
} MV300_PORT;

/* Controller status register bits */
#define CSR_MASK	0x3f		/* controller status register mask */
#define CSR_BERR	0x20		/* bus error */
#define CSR_SYSFAIL	0x10		/* SYSFAIL LED is lit */
#define CSR_BCRDN	0x08		/* byte count register done */
#define CSR_FULL	0x04		/* FIFO full */
#define CSR_EODNE	0x02		/* EOS/EOI received */
#define CSR_EMPTY	0x01		/* FIFO empty */

/* Uniline command bits */
#define UCR_DMAW	0x01		/* DMA write */
#define UCR_EOS		0x02		/* Enable EOS compare */
#define UCR_DMAR	0x04		/* DMA read  */
#define UCR_WFAIL	0x10		/* Sysfail LED set/clear */
#define UCR_CLEAR	0x20		/* Clear BCRDN + BERR */

/* Tms9914 interrupt bits */
#define IM0_MASK	0x3f
#define IM0_BI		0x20
#define IM0_BO		0x10
#define IM0_END		0x08
#define IM0_SPAS	0x04
#define IM0_RLC		0x02
#define IM0_MAC		0x01
#define IM1_GETS	0x80
#define IM1_UUCG	0x40
#define IM1_UACG	0x20
#define IM1_APT		0x10
#define IM1_DCAS	0x08
#define IM1_MA		0x04
#define IM1_SRQ		0x02
#define IM1_IFC		0x01

/* Tms9914 address_status_bits */
#define ASR_REM		0x80
#define ASR_ATN		0x40
#define ASR_LLO		0x20
#define ASR_LPAS	0x10
#define ASR_TPAS	0x08
#define ASR_LADS	0x04
#define ASR_TADS	0x02
#define ASR_ULPA	0x01

/* Tms9914 bus_state register */
#define BSR_ATN		0x80
#define BSR_DAV		0x40
#define BSR_NDAC	0x20
#define BSR_NRFD	0x10
#define BSR_EOI		0x08
#define BSR_SRQ		0x04
#define BSR_IFC		0x02
#define BSR_REN		0x01

/* Tms9914 aux_command functions */

#define CMD_SWRST_SET	 0X80	   /* Software Reset */
#define CMD_SWRST_CLR	 0X00
#define CMD_DACR	 0X01	   /* Release DAC holdoff */
#define CMD_DAI_SET	 0X93	   /* Disable all interrupts */
#define CMD_DAI_CLR	 0X13
#define CMD_FEOI	 0X08	   /* Send EOI with next byte */
#define CMD_FGET_PULSE   0X06	   /* Pulse TR output line */
#define CMD_FGET_SET	 0X86	   /* Force TR output line */
#define CMD_FGET_CLR	 0X06
#define CMD_GTS		 0X0B	   /* Go to standby */
#define CMD_HDFA_SET	 0X83	   /* Holdoff on all data */
#define CMD_HDFA_CLR	 0X03
#define CMD_HDFE_SET	 0X84	   /* Holdoff on EOI only */
#define CMD_HDFE_CLR	 0X04
#define CMD_LON_SET	 0X89	   /* Listen only */
#define CMD_LON_CLR	 0X09
#define CMD_NBAF	 0X05	   /* New byte available false */
#define CMD_PTS		 0X14	   /* Pass through next secondary */
#define CMD_RHDF	 0X02	   /* Release RFD holdoff */
#define CMD_RLC		 0X12	   /* Release control */
#define CMD_RPP_SET	 0X8E	   /* Request parallel poll */
#define CMD_RPP_CLR	 0X0E
#define CMD_RQC		 0X11	   /* Request control */
#define CMD_RSV2_SET	 0X98	   /* Request service bit 2 */
#define CMD_RSV2_CLR	 0X18
#define CMD_RTL_SET	 0X87	   /* Return to local */
#define CMD_RTL_CLR	 0X07
#define CMD_SHDW_SET	 0X96	   /* Shadow handshake */
#define CMD_SHDW_CLR	 0X16
#define CMD_ADDR_VALID	 0x81	   /* Secondary address valid */
#define CMD_ADDR_INVALID 0x01
#define CMD_SIC_SET	 0X8F	   /* Send interface clear */
#define CMD_SIC_CLR	 0X0F
#define CMD_SRE_SET	 0X90	   /* Send remote enable */
#define CMD_SRE_CLR	 0X10
#define CMD_STD1_SET	 0X95	   /* Short T1 settling time */
#define CMD_STD1_CLR	 0X15
#define CMD_TCA		 0X0C	   /* Take control asynchronously */
#define CMD_TCS		 0X0D	   /* Take control synchronously */
#define CMD_TON_SET	 0X8A	   /* Talk only */
#define CMD_TON_CLR	 0X0A
#define CMD_VSTD1_SET	 0X97	   /* Very short T1 delay */
#define CMD_VSTD1_CLR	 0X17

/* Tms9914 serial poll request bit*/
#define SPR_RSV1	0x40

/* BIM MC68153  control register bits */
#define BIM_DIS		0
#define BIM_IRQ(X)	(X)		/* level */
#define BIM_IRAC	0x08		/* auto-clear */
#define BIM_IRE		0x10		/* interrupt enable */
#define BIM_XIN		0x20		/* external internal */
#define BIM_FAC		0x40		/* flag auto-clear */
#define BIM_FLAG	0x80		/* flag */

/*========================= Local Macros =====================================*/
#define MAR0(X)		((UINT_4)(X) >>  0)
#define MAR1(X)		((UINT_4)(X) >>  8)
#define MAR2(X)		((UINT_4)(X) >> 16)
#define BCNT0(X)	(((~((UINT) (X))) >> 0) & 0xff)
#define BCNT1(X)	(((~((UINT) (X))) >> 8) & 0xff)
#define SYS_CTRL(PB)	(~((MV300_PORT *) PB)->tms.r.addr_sw1 & 0x80)

/*========================= Local Function Prototypes ========================*/
LOCAL VOID   brdInt ();
LOCAL VOID   brdOffline ();
LOCAL VOID   brdOnline ();
LOCAL VOID   brdTimeout ();
LOCAL STATUS brdWait ();
LOCAL VOID   busInit ();

/*========================= Local Variables ==================================*/

/*========================= Exported Functions ===============================*/

/*------------------------- Create and Forget --------------------------------*/
/*-----------------------------------------------------------------------------
 * Mv300BrdCreate (pBrd, host, ioBase, intNum, intLevel, pConfig) : status
 *
 * Purpose:
 *  Create and initialize the MVME300 board and control structure
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  pBrd	: MV300_BRD_ID	- board control structure ID
 *  host	: STRING	- pointer to hostname (for RPC compatibility)
 *  ioBase	: ADDR		- base address of the MVME300 board
 *  intNum	: INT		- interrupt vector number
 *  intLevel	: INT		- interrupt level
 *  pConfig     : PMV300_CONFIG - pointer to board configuration structure
 * Description:
 *  Create and initialize MVME300_BRD structure and initialize board.
 *  At present, the priAddr is derived from on-board dip switches and NOT
 *  from the pConfig structure. secAddr is also always 0 (zero).
 */
STATUS Mv300BrdCreate (pBrdId, host, ioBase, intNum, intLevel, pConfig)
     MV300_BRD_ID* pBrdId;
     STRING	   host;
     ADDR 	   ioBase;
     INT	   intNum;
     INT	   intLevel;
     PMV300_CONFIG pConfig;
{
  FAST MV300_PORT*  pPort  = (MV300_PORT*) ioBase;
  FAST MV300_BRD_ID   brdId  = (MV300_BRD_ID) NULL;
  INT  		    test;
  
  /* probe for MV300 hardware */
  if (vxMemProbe (&(pPort->mar0), READ, 1, &test) != OK)
    {
      *pBrdId = brdId;
      return (ERROR_EBRD);
    }

  /* Else try to create module control structure */
  if ((brdId = (MV300_BRD_ID) malloc (sizeof (MV300_BRD))) == NULL)
    {
      *pBrdId = brdId;
      return (ERROR_EMEM);
    }

  /* Initialize module control structure */
  brdId->port		  = (ADDR) pPort;
  brdId->wDogId		  = wdCreate ();
  brdId->brdSemId	  = semCreate ();
  brdId->intSemId	  = semCreate ();
  brdId->intNumber	  = intNum;
  brdId->intLevel	  = intLevel;
  brdId->config.timeout	  = pConfig->timeout;
  brdId->config.flags	  = pConfig->flags;
  brdId->config.priAddr	  = pPort->tms.r.addr_sw1 & 0x1f;
  brdId->config.secAddr	  = 0;
  brdId->config.reosChar  = pConfig->reosChar;
  brdId->config.weosChar  = pConfig->weosChar;
  brdId->state		  = 0;
  brdId->status.brd_stat  = 0;
  brdId->status.int0_stat = 0;
  brdId->status.int1_stat = 0;
  brdId->status.addr_stat = 0;

  /* connect to interrupt */
  intConnect (INUM_TO_IVEC (intNum), brdInt, (INT) brdId);
  sysIntEnable (intLevel);

  /* Initialize board */
  pPort->gcrst	 = 0;		/* reset board */

  /* Place board online */
  brdOnline (brdId);

  /* initialize mutex semaphore */
  semGive (brdId->brdSemId);

  /* Return structure id */
  *pBrdId = brdId;
  return (OK);
}/* Mv300BrdCreate */

/*-----------------------------------------------------------------------------
 * Mv300BrdForget (brdId) : status
 *
 * Purpose:
 *  Offline and free memory
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure
 * Description:
 *  Place board offline and free MV300_BRD structure.
 * NOTE:
 *  Care should be taken to avoid calling this function if other tasks
 *  have been given this board ID. Generally, this function should never
 *  need to be called. Mv300BrdCreate would be called during system
 *  initialization, and all tasks which use the interface would be passed
 *  the board_id. This function would only be called if a graceful shutdown
 *  is required.
 */
STATUS Mv300BrdForget (brdId)
     MV300_BRD_ID brdId;
{
  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);
  
  /* place board offline */
  brdOffline (brdId);

  /* free structure and return */
  semGive (brdId->brdSemId);
  semDelete (brdId->brdSemId);
  semDelete (brdId->intSemId);
  return (free ((ADDR) brdId));
}/* Mv300BrdForget*/

/*------------------------- Board operations ---------------------------------*/
/*-----------------------------------------------------------------------------
 * Mv300BrdAtn (brdId, state) : status
 *
 * Purpose:
 *  Raise/Lower ATN
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure.
 *  state	: BOOL		- TRUE = Raise ATN, FALSE = Clear ATN.
 * Description:
 *  If board is CIC then raise ATN when state is TRUE and lower ATN
 *  when state is FALSE.
 */
STATUS Mv300BrdAtn (brdId, state)
     MV300_BRD_ID brdId;
     BOOL	  state;
{
  FAST MV300_PORT*  pPort = NULL;
  STATUS            status = OK;

  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);
  
  pPort = (MV300_PORT*) brdId->port;

  /* Initialize interrupt mask */
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;

  /* if CIC then write to auxillary command register */
  if (brdId->state & BRD_CIC)
    {
      /* ATN On? */
      if (state != FALSE)
	{
	  /* take control */
	  if (!(pPort->tms.r.addr_stat & ASR_ATN))
	    {
	      pPort->tms.w.int0_mask = IM0_BO;
	      pPort->tms.w.aux_cmd = CMD_TCA;
	      status = brdWait (brdId);
	      pPort->tms.w.int0_mask = 0;
	    }
	}
      else
	{
	  pPort->tms.w.aux_cmd = CMD_GTS;	/* go to standby */
	  status = OK;
	}
    }
  else
    {
      status = ERROR_ECIC;
    }

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (status);
} /*Mv300BrdAtn*/

/*-----------------------------------------------------------------------------
 * Mv300BrdCommand (brdId, pBuf, bufLen, pBytesWritten) : status
 *
 * Purpose:
 *  Send multiline commands from buffer
 * Returns:
 *  status		: STATUS	- OK if no problems
 * Arguments:	     
 *  brdId		: MV300_BRD_ID	- pointer to board control structure
 *  pBuf		: BYTE*		- pointer to command array
 *  bufLen      	: UINT		- number of bytes in command array
 *  pBytesWritten	: UINT*		- pointer to variable for number
 *					  of bytes written
 * Description:
 *  Using the board configuration parameter for timeouts, send multiline
 *  commands from buffer pBuf and return number of bytes written in variable
 *  pointed to by pBytesWritten. If timeout = 0, then watchdog is disabled.
 *  If board is not controller in charge, return error.
 */
STATUS Mv300BrdCommand (brdId, pBuf, bufLen, pBytesWritten)
     MV300_BRD_ID brdId;
     BYTE*	  pBuf;
     UINT	  bufLen;
     UINT*	  pBytesWritten;
{
  FAST MV300_PORT*  pPort;
  STATUS	    status = OK;
  INT	    	    index  = 0;
  
  /* test for valid board id */
  if (brdId == NULL)
    {
      *pBytesWritten = 0;
      return (ERROR_EBID);
    }

  /* fast pointer to port */
  pPort = (MV300_PORT*) brdId->port;

  /* grab control and start watchdog if timeout non-zero */
  semTake (brdId->brdSemId);
  brdId->state &= ~BRD_TIMEOUT;
  if (brdId->config.timeout != 0)
    wdStart (brdId->wDogId, brdId->config.timeout, brdTimeout, (INT) brdId);

  /* initialize interrupt masks and clear output buffer */
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;
  pPort->tms.w.aux_cmd   = CMD_NBAF;

  /* test for controller-in-charge */
  if (!(brdId->state & BRD_CIC))
    {
      *pBytesWritten = 0;
      status = ERROR_ECIC;
    }
  else /* send multiline commands */
    {
      /* take control */
      pPort->tms.w.int0_mask = IM0_BO;
      if (!(pPort->tms.r.addr_stat & ASR_ATN))
	{
	  pPort->tms.w.aux_cmd = CMD_TCA;
	  status = brdWait (brdId);
	}
      /* send commands */
      for (index = 0; (status == OK) && (index < bufLen); index++)
	{
	  pPort->tms.w.data_out  = pBuf[index];
	  status = brdWait (brdId);
	}
      *pBytesWritten = index;
      /* release the bus */
      pPort->tms.w.int0_mask = 0;
      pPort->tms.w.aux_cmd = CMD_GTS;
    }

  /* clear watchdog and test for timeout */
  wdCancel (brdId->wDogId);
  /* semaphore should only be set if timeout  before reaching this point */
  if (semClear (brdId->intSemId) == OK)	/* test and clear */
    {
      status = ERROR_ETMW;
    }
  
 /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (status);
}/*Mv300BrdCommand*/

/*-----------------------------------------------------------------------------
 * Mv300BrdConfRd (brdId, pConf) : status
 *
 * Purpose:
 *  Read board configuration
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure.
 *  pConf	: PMV300_CONFIG	- pointer to board configuration structure.
 * Description:
 *  Read the current device configuration parameters.
 * NOTE:
 *  This function needs to grab brdSemId to protect data region during copy.
 */
STATUS Mv300BrdConfRd (brdId, pConf)
     MV300_BRD_ID brdId;
     PMV300_CONFIG  pConf;
{
  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);
  
  /* copy board configuration */
  pConf->timeout  = brdId->config.timeout;
  pConf->flags	  = brdId->config.flags;
  pConf->priAddr  = brdId->config.priAddr;
  pConf->secAddr  = brdId->config.secAddr;
  pConf->reosChar = brdId->config.reosChar;
  pConf->weosChar = brdId->config.weosChar;

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (OK);
} /*Mv300BrdConfRd*/

/*-----------------------------------------------------------------------------
 * Mv300BrdConfWrt (brdId, pConf) : status
 *
 * Purpose:
 *  Write board configuration
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure.
 *  pConf	: PMV300_CONFIG	- pointer to board configuration structure.
 * Description:
 *  Set board configuration parameters according to values passed in pConf
 * NOTE:
 *  Currently, the priAddr and secAddr elements are NOT updated.
 *  priAddr is derived from the board switch settings and
 *  secAddr is always 0 (zero).
 */
STATUS Mv300BrdConfWrt (brdId, pConf)
     MV300_BRD_ID brdId;
     PMV300_CONFIG  pConf;
{
  FAST MV300_PORT*  pPort = NULL;

  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);
  
  pPort = (MV300_PORT*) brdId->port;

  /* update board configuration */
  brdId->config.timeout  = pConf->timeout;
  brdId->config.flags	 = pConf->flags;
  brdId->config.priAddr	 = pPort->tms.r.addr_sw1 & 0x1f;
  brdId->config.secAddr	 = 0;
  brdId->config.reosChar = pConf->reosChar;
  brdId->config.weosChar = pConf->weosChar;

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (OK);
} /*Mv300BrdConfWrt*/

/*-----------------------------------------------------------------------------
 * Mv300BrdLocal (brdId, state) : status
 *
 * Purpose:
 *  Set or Clear "return-to-local" request.
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure.
 *  state	: BOOL		- TRUE = set rtl; FALSE = Clear rtl.
 * Description:
 *  If state = TRUE, send a "return to local" (rtl) message to the board.
 *  This will clear an internal remote status condition if it is set
 *  and if the internal lockout condition is not set.
 *  These conditions can be checked with the Mv300BrdStatus function.
 * NOTE:
 */
STATUS Mv300BrdLocal (brdId, state)
     MV300_BRD_ID brdId;
     BOOL	  state;
{
  FAST MV300_PORT*  pPort = NULL;

  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);
  
  pPort = (MV300_PORT*) brdId->port;

  /* send rtl command */
  if (state != FALSE)
    {
      pPort->tms.w.aux_cmd = CMD_RTL_SET;
    }
  else
    {
      pPort->tms.w.aux_cmd = CMD_RTL_CLR;
    }

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (OK);
} /*Mv300BrdLocal*/

/*-----------------------------------------------------------------------------
 * Mv300BrdOffline (brdId) : status
 *
 * Purpose:
 *  Place board offline
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure
 * Description:
 *  Grab board mutex and then call local function to place board offline.
 */
STATUS Mv300BrdOffline (brdId)
     MV300_BRD_ID       brdId;
{
  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);
  
  /* place board offline */
  brdOffline (brdId);

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (OK);
}/*Mv300BrdOffline*/

/*-----------------------------------------------------------------------------
 * Mv300BrdOnline (brdId) : status
 *
 * Purpose:
 *  Place the board online.
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID		- pointer to board control structure
 * Description:
 *  Grab board mutex and then call local function to place board online.
 */
STATUS Mv300BrdOnline (brdId)
     MV300_BRD_ID 	brdId;
{
  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);

  /* call local online routine */
  brdOnline (brdId);

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (OK);
}/*Mv300BrdOnline*/

/*-----------------------------------------------------------------------------
 * Mv300BrdPPoll (brdId, pResult) : status
 *
 * Purpose:
 *  Perform a parallel poll on the bus
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure.
 *  pResult	: BYTE*		- pointer to result.
 * Description:
 *  If board is CIC, perform a parallel poll and return result in pResult.
 */
STATUS Mv300BrdPPoll (brdId, pResult)
     MV300_BRD_ID brdId;
     BYTE*        pResult;
{
  FAST MV300_PORT*  pPort = NULL;
  STATUS            status = OK;

  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control and start watchdog */
  semTake (brdId->brdSemId);
  brdId->state &= ~BRD_TIMEOUT;
  /* use board default timeout */
  if (brdId->config.timeout != 0)
    wdStart (brdId->wDogId, brdId->config.timeout, brdTimeout, (INT)brdId);
  
  pPort = (MV300_PORT*) brdId->port;

  /* initialize masks */
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;

  if (!(brdId->state & BRD_CIC))
    {
      *pResult = 0;
      status   = ERROR_ECIC;
    }
  else /* parallel poll */
    {
      /* take control */
      pPort->tms.w.int0_mask = IM0_BO;
      if (!(pPort->tms.r.addr_stat & ASR_ATN))
	{
	  pPort->tms.w.aux_cmd = CMD_TCA;
	  status = brdWait (brdId);
	}
      if (status == OK)
	  {
	    /* request parallel poll, read status, unrequest */
	    pPort->tms.w.aux_cmd = CMD_RPP_SET;
	    taskDelay (1); 			/* wait at least 2us */
	    *pResult = pPort->tms.r.cmd_pass;	/* read pass-thru register */
	    pPort->tms.w.aux_cmd = CMD_RPP_CLR;
	  }
      /* release the bus */
      pPort->tms.w.int0_mask = 0;
      pPort->tms.w.aux_cmd = CMD_GTS;
    }

  /* clear watchdog and test for timeout */
  /* intSemId should only be set if timeout before reaching this point */
  wdCancel (brdId->wDogId);
  if (semClear (brdId->intSemId) == OK)	/* test and clear */
    {
      status = ERROR_ETMW;
    }

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (status);
} /*Mv300BrdPPoll*/

/*-----------------------------------------------------------------------------
 * Mv300BrdPPollByte (brdId, ppByte) : status
 *
 * Purpose:
 *  Write to parallel poll register
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID		- pointer to board control structure
 *  ppByte	: BYTE			- Parallel poll data byte
 * Description:
 */
STATUS Mv300BrdPPollByte (brdId, ppByte)
     MV300_BRD_ID       brdId;
     BYTE		ppByte;
{
  FAST MV300_PORT*  pPort;

  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);
  /* else grab control */
  semTake (brdId->brdSemId);

  /* get fast port address */
  pPort = (MV300_PORT*) brdId->port;

  /* write parallel poll register */
  pPort->tms.w.par_poll = ppByte;
  
  /* release control of board and return */
  semGive (brdId->brdSemId);
  return (OK);
}/*Mv300BrdPPollByte*/

/*-----------------------------------------------------------------------------
 * Mv300BrdRead (brdId, pBuf, bufLen, pBytesRead) : status
 *
 * Purpose:
 *  Read device data from controller data_in port
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  brdId		: MV300_BRD_ID	 - pointer to board control structure
 *  pBuf		: BYTE*		 - pointer to data byte array
 *  bufLen		: UINT		 - number of bytes in data array
 *  pBytesRead		: UINT*		 - pointer to variable for number
 *					   of bytes read.
 * Description:
 *  If board is CIC, initiate a lon (listen-only) and read from input register.
 *  If not CIC, wait for either input or, if enabled, timeout.
 */
STATUS Mv300BrdRead (brdId, pBuf, bufLen, pBytesRead)
     MV300_BRD_ID         brdId;
     BYTE*		  pBuf;
     UINT		  bufLen;
     UINT*		  pBytesRead;
{
  FAST MV300_PORT*  pPort = NULL;
  STATUS	    status = OK;
  UINT	    	    byteCount = 0;
  BYTE *	    busAdrs = 0;
  BYTE		    dummy;

  /* test for valid board id */
  if (brdId == NULL)
    {
      *pBytesRead = 0;
      return (ERROR_EBID);
    }
  
  /* grab control and start watchdog */
  semTake (brdId->brdSemId);
  brdId->state &= ~BRD_TIMEOUT;
  if (brdId->config.timeout != 0)
    wdStart (brdId->wDogId, brdId->config.timeout, brdTimeout, (INT) brdId);
  
  /* initialize fast pointers */
  pPort = (MV300_PORT*) brdId->port;
  
  /* disable tms interrupts */
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;
  
  /* set up for listening */
  /* disable all holdoff except data RFD holdoff */
  pPort->tms.w.aux_cmd = CMD_HDFA_CLR;	/* disable holdoff on all data */
  pPort->tms.w.aux_cmd = CMD_HDFE_CLR;	/* disable holdoff on EOI data */
  pPort->tms.w.aux_cmd = CMD_DACR;	/* clear any DAC holdoff */

  /* if CIC then place in LADS state with a lon command */
  if (brdId->state & BRD_CIC)
    {
      pPort->tms.w.aux_cmd = CMD_LON_SET;    /* enable listener */
      pPort->tms.w.aux_cmd = CMD_GTS;	     /* make sure ATN is false */
    }
  /* else, must wait for bus controller to initiate LADS state */
  
  /* if status is OK, do either dma or programmed read */
  if (status == OK)
    {
      /* read data either by dma or programmed read */
      if (brdId->config.flags & MV300_CONFIG_V_DMA)
	{ /* BEGIN DMA READ */
	  /* get vme bus address */
	  if (sysLocalToBusAdrs (VME_AM_STD_SUP_DATA, pBuf, &busAdrs) != OK)
	    {
	      byteCount = 0;
	      status = ERROR_EARG;
	    }
	  else /* valid vme bus address */
	    {
	      pPort->mar0 = MAR0 (busAdrs);
	      pPort->mar1 = MAR1 (busAdrs);
	      pPort->mar2 = MAR2 (busAdrs);
	      
	      /* load byte count register */
	      byteCount = bufLen;
	      pPort->bcr0 = BCNT0 (byteCount);
	      pPort->bcr1 = BCNT1 (byteCount);
	      
	      pPort->eoscr = brdId->config.reosChar;
	      
	      /* start dma read */
	      pPort->tms.w.int0_mask = IM0_END;
	      pPort->ucr = (brdId->config.flags & MV300_CONFIG_V_REOS) ?
		UCR_DMAW | UCR_EOS : UCR_DMAW;
	      
	      /* wait for interrupt */
	      status = brdWait (brdId);
	      
	      /* get number of bytes actually read */
	      byteCount = ((BYTE*) ((pPort->mar0 << 0) +
				    (pPort->mar1 << 8) +
				    (pPort->mar2 << 16))) - busAdrs - 1;

	      /* get last byte in input register */
	      pBuf[byteCount++] = pPort->tms.r.data_in;
	    }
	  
	} /* END DMA READ */
      else
	{ /* BEGIN PROGRAMMED READ */
	  pPort->tms.w.int0_mask = IM0_BI;
	  for (byteCount = 0; byteCount < bufLen; byteCount++)
	    {
	      /* get next byte */
	      if ((status = brdWait (brdId)) != OK)
		break;
	      pBuf[byteCount] = pPort->tms.r.data_in;
	      /* EOS? */
	      if ((brdId->config.flags & MV300_CONFIG_V_REOS) &&
		  (pBuf[byteCount] == brdId->config.reosChar))
		{
		  byteCount++;
		  break;
		}
	      /* EOI? */
	      if (brdId->status.int0_stat & IM0_END)
		{
		  byteCount++;
		  break;
		}
	    }
	  pPort->tms.w.int0_mask = 0;
	} /* END PROGRAMMED READ */
    } /* END READ */
  
  /* turn off listener and flush input buffer */
  if (brdId->state & BRD_CIC)
    {
      pPort->tms.w.aux_cmd = CMD_LON_CLR;
      dummy = pPort->tms.r.data_in;
    }
  
  /* clear watchdog and test for timeout */
  /* intSemId should only be set if timeout  before reaching this point */
  wdCancel (brdId->wDogId);
  if (semClear (brdId->intSemId) == OK)	/* test and clear */
    {
      status = ERROR_ETMW;
    }
 
 /* restore control of board and return */
  semGive (brdId->brdSemId);
  *pBytesRead = byteCount;
  return (status);
}/*Mv300BrdRead*/

/*-----------------------------------------------------------------------------
 * Mv300BrdRemote (brdId, state) : status
 *
 * Purpose:
 *  Set/Clear remote enable.
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure.
 *  state	: BOOL		- TRUE = set remote enable; FALSE = clear.
 * Description:
 *  If board is CIC, drive REN according to value of state.
 * NOTE:
 */
STATUS Mv300BrdRemote (brdId, state)
     MV300_BRD_ID brdId;
     BOOL         state;
{
  FAST MV300_PORT*  pPort = NULL;
  STATUS            status = OK;

  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);
  
  pPort = (MV300_PORT*) brdId->port;

  if (!(brdId->state & BRD_CIC))
    {
      status = ERROR_ECIC;
    }
  else /* set REN according to state */
    {
      if (state != FALSE)
	{
	  pPort->tms.w.aux_cmd = CMD_SRE_SET;
	}
      else
	{
	  pPort->tms.w.aux_cmd = CMD_SRE_CLR;
	}
      status = OK;
    }

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (status);
} /*Mv300BrdRemote*/

/*-----------------------------------------------------------------------------
 * Mv300BrdRequest (brdId, state, sPollByte) : status
 *
 * Purpose:
 *  Set/Clear request service
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure.
 *  state	: BOOL		- Set rsv if TRUE; else clear rsv
 *  sPollByte	: BYTE		- serial poll response byte.
 * Description:
 *  Request service from CIC by asserting SRQ.
 *  When the CIC performs a serial poll, the board will respond
 *  with sPollByte | SPR_RSV1 (bit 7 set).
 * NOTE:
 *  SPR_RSV1 is not set explicitly. The CMD_RSV2_SET auxillary command
 *  toggles this bit on/off during the serial poll response.
 *  This function should not normally need to be called with state=FALSE.
 *  RQS is normally reset after the device is polled. If however you need
 *  to reset RQS before the device is polled, then the function can be
 *  called with state=FALSE and the RQS line will be reset.
 */
STATUS Mv300BrdRequest (brdId, state, sPollByte)
     MV300_BRD_ID brdId;
     BOOL	  state;
     BYTE	  sPollByte;
{
  FAST MV300_PORT*  pPort = NULL;

  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);
  
  pPort = (MV300_PORT*) brdId->port;

  /* load serial poll response byte and request service */
  if (state != FALSE)
    {
      pPort->tms.w.ser_poll = sPollByte & ~SPR_RSV1;	/* don`t set rsv1 */
      pPort->tms.w.aux_cmd = CMD_RSV2_SET;
    }
  else
    {
      pPort->tms.w.aux_cmd = CMD_RSV2_CLR;
    }

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (OK);
} /*Mv300BrdRequest*/

/*-----------------------------------------------------------------------------
 * Mv300BrdSendIfc (brdId) : status
 *
 * Purpose:
 *  Send IFC command
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure.
 * Description:
 *  If the board is System Controller, send IFC.
 * NOTE:
 */
STATUS Mv300BrdSendIfc (brdId)
     MV300_BRD_ID brdId;
{
  FAST MV300_PORT*  pPort = NULL;
  STATUS            status = OK;

  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);
  
  pPort = (MV300_PORT*) brdId->port;

  /* initialize interrupt masks */
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;

  /* if system controller: send IFC, wait 1 tick, clear IFC */
  if (SYS_CTRL (pPort))
    {
      pPort->tms.w.aux_cmd = CMD_SIC_SET;
      taskDelay (1); /* wait at least 100us */
      pPort->tms.w.aux_cmd = CMD_SIC_CLR;
      brdId->state |= BRD_SC;
      /* wait for CIC and initialize state bits */
      pPort->tms.w.int0_mask = IM0_BO;
      if ((status = brdWait (brdId)) == OK)
	{
	  brdId->state |= BRD_CIC;
	}
      pPort->tms.w.int0_mask = 0;
      pPort->tms.w.aux_cmd   = CMD_GTS;
    }
  else /* signal error, must be system controller to use this function */
    {
      brdId->state &= ~BRD_SC;
      status = ERROR_ESAC;
    }

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (status);
} /*Mv300BrdSendIfc*/

/*-----------------------------------------------------------------------------
 * Mv300BrdSPollByte (brdId, spByte) : status
 *
 * Purpose:
 *  Write to serial poll register.
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID		- pointer to board control structure
 *  spByte	: BYTE			- serial poll byte
 * Description:
 *  Write to serial poll register masking off bit 7 (SPR_RSV1)
 */
STATUS Mv300BrdSPollByte (brdId, spByte)
     MV300_BRD_ID       brdId;
     BYTE		spByte;
{
  FAST MV300_PORT*  pPort;

  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);

  /* get fast port address */
  pPort = (MV300_PORT*) brdId->port;

  /* write to serial poll register */
  pPort->tms.w.ser_poll = (spByte & ~SPR_RSV1);
  
  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (OK);
}/*Mv300BrdSPollByte*/

/*-----------------------------------------------------------------------------
 * Mv300BrdStatus (brdId, pStatus) : status
 *
 * Purpose:
 *  Get current board status
 * Returns:
 *  status	: STATUS		- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID		- pointer to brdId control structure
 *  pStatus	: PMV300_STATUS 	- pointer to board status structure
 * Description:
 *  load MV300_STATUS structure with board status register info
 */
STATUS Mv300BrdStatus (brdId, pStatus)
     MV300_BRD_ID	brdId;
     MV300_STATUS*	pStatus;
{
  FAST MV300_BRD_STATUS* pBrdStatus = (MV300_BRD_STATUS*) pStatus;
  FAST MV300_PORT*	 pPort;

  /* test for valid board id */
  if (brdId == NULL)
    return (ERROR_EBID);

  /* else grab control */
  semTake (brdId->brdSemId);

  /* get fast port address */
  pPort = (MV300_PORT*) brdId->port;
  if (pStatus != NULL)
    {
      /* Get status */
      pBrdStatus->brd_stat  = pPort->csreg & CSR_MASK;
      pBrdStatus->int0_stat = pPort->tms.r.int0_stat & IM0_MASK;
      pBrdStatus->int1_stat = pPort->tms.r.int1_stat;
      pBrdStatus->addr_stat = pPort->tms.r.addr_stat;
      if (brdId->state & BRD_TIMEOUT)
	*pStatus |= MV300_STATUS_V_TIMEO;
      if (brdId->state & BRD_SC)
	*pStatus |= MV300_STATUS_V_SC;
      if (brdId->state & BRD_CIC)
	*pStatus |= MV300_STATUS_V_CIC;
    }
  
  /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (OK);
}/*Mv300BrdStatus*/

/*-----------------------------------------------------------------------------
 * Mv300BrdWrite (brdId, pBuf, bufLen, pBytesWritten) : status
 *
 * Purpose:
 *  Write to data_out register
 * Returns:
 *  status		: STATUS	- OK if no problems
 * Arguments:			
 *  brdId		: MV300_BRD_ID 	 - pointer to board control structure
 *  pBuf		: BYTE*		 - pointer to data byte array
 *  bufLen		: UINT		 - number of bytes in data array
 *  pBytesWritten	: UINT*		 - pointer to variable for
 *					   number of bytes written.
 * Description:
 *  If CIC, send MTA and force TADS/TACS and start talking; else wait for
 *  bus controller to put unit into TADS/TACS and then start talking.
 *  In this latter case it is probably best to disable timeouts unless
 *  you are certain that the unit is already in TADS/TACS.
 */
STATUS Mv300BrdWrite (brdId, pBuf, bufLen, pBytesWritten)
     MV300_BRD_ID       brdId;
     BYTE*		pBuf;
     UINT		bufLen;
     UINT*		pBytesWritten;
{
  FAST MV300_PORT*  pPort = NULL;
  STATUS	    status = OK;
  INT		    index = 0;
  UINT		    byteCount = 0;
  BYTE *	    busAdrs = 0;
  
  /* test for valid board id */
  if (brdId == NULL)
    {
      *pBytesWritten = 0;
      return (ERROR_EBID);
    }
  /* else grab control and start watchdog */
  semTake (brdId->brdSemId);
  brdId->state &= ~BRD_TIMEOUT;
  if (brdId->config.timeout != 0)
    wdStart (brdId->wDogId, brdId->config.timeout, brdTimeout, (INT) brdId);
  
  /* initialize fast pointer */
  pPort = (MV300_PORT*) brdId->port;

  /* clear out data_out register */
  pPort->tms.w.aux_cmd = CMD_NBAF;
  
  /* initialize interrupt masks */
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;

  /* wait till in TADS */
  if (!(brdId->state & BRD_CIC))
    {
      /* wait for controller  to initiate TADS/TACS state */
      pPort->tms.w.int0_mask = IM0_BO;
      status = brdWait (brdId);
      pPort->tms.w.int0_mask = 0;
    }
  else
    {
      /* take control */
      pPort->tms.w.int0_mask = IM0_BO;
      if (!(pPort->tms.r.addr_stat & ASR_ATN))
	{
	  pPort->tms.w.aux_cmd = CMD_TCA;
	  status = brdWait (brdId);
	}
      if (status == OK)
	{
	  pPort->tms.w.data_out = MV300_BRD_K_GPIB_UNT; /* UNTALK */
	  status = brdWait (brdId);
	}
      if (status == OK)
	{
	  pPort->tms.w.aux_cmd = CMD_TON_SET;	    /* goto TADS/TACS */
	}
      pPort->tms.w.aux_cmd = CMD_GTS;		    /* goto standby */
      /* wait for TADS/TACS state */
      if (status == OK)
	{
	  status = brdWait (brdId);
	}
      pPort->tms.w.int0_mask = 0;
    }

  /* do either dma or programmed write */
  if (status == OK)
    {
      byteCount = (brdId->config.flags & MV300_CONFIG_V_WEOS) ?
	bufLen : bufLen - 1;
      if (brdId->config.flags & MV300_CONFIG_V_DMA)
	{ /* BEGIN DMA WRITE */
	  /* get vme bus address */
	  if (sysLocalToBusAdrs (VME_AM_STD_SUP_DATA, pBuf, &busAdrs) != OK)
	    {
	      *pBytesWritten = 0;
	      status = ERROR_EARG;
	    }
	  else
	    {
	      pPort->mar0 = MAR0 (busAdrs);
	      pPort->mar1 = MAR1 (busAdrs);
	      pPort->mar2 = MAR2 (busAdrs);
	      
	      /* load byte count register */
	      pPort->bcr0 = BCNT0 (byteCount);
	      pPort->bcr1 = BCNT1 (byteCount);
	      
	      /* start dma write */
	      pPort->ucr = UCR_DMAR;
	      
	      /* wait for interrupt */
	      status = brdWait (brdId);
	      
	      /* get number of bytes actually read */
	      byteCount = ((BYTE*) ((pPort->mar0 << 0) +
				    (pPort->mar1 << 8) +
				    (pPort->mar2 << 16))) - busAdrs;
	    }
	} /* END DMA WRITE */
      else /* Programmed IO */
	{ /* BEGIN PROGRAMMED WRITE */
	  /* send all but last byte */
	  pPort->tms.w.int0_mask = IM0_BO;
	  for (index = 0; (status == OK) && (index < byteCount); index++)
	    {
	      pPort->tms.w.data_out = pBuf[index];
	      status = brdWait (brdId);
	    }
	  byteCount = index;
	  pPort->tms.w.int0_mask = 0;
	} /* END PROGRAMMED WRITE */
      /* if OK so far, send last byte with EOI */
      if (status == OK)
	{
	  pPort->tms.w.int0_mask = IM0_BO;
	  pPort->tms.w.aux_cmd = CMD_FEOI;
	  if (brdId->config.flags & MV300_CONFIG_V_WEOS)
	    {
	      pPort->tms.w.data_out = brdId->config.weosChar;
	      status = brdWait (brdId);
	    }
	  else
	    {
	      pPort->tms.w.data_out = pBuf[bufLen - 1];
	      if ((status = brdWait (brdId)) == OK)
		byteCount++;
	    }
	  pPort->tms.w.int0_mask = 0;
	}
    } /*  end of write*/

  /* clean up */
  if (brdId->state & BRD_CIC)
    {
      pPort->tms.w.int0_mask = IM0_BO;
      pPort->tms.w.aux_cmd = CMD_TCA;
      if (status == OK)
	status = brdWait (brdId);
      pPort->tms.w.data_out = MV300_BRD_K_GPIB_UNL;
      if (status == OK)
	status = brdWait (brdId);
      pPort->tms.w.aux_cmd = CMD_TON_CLR;
      pPort->tms.w.int0_mask = 0;
      pPort->tms.w.aux_cmd = CMD_GTS;
    }

  /* clear watchdog and test for timeout */
  /* intSemId should only be set if timeout  before reaching this point */
  wdCancel (brdId->wDogId);
  if (semClear (brdId->intSemId) == OK)	/* test and clear */
    {
      status = ERROR_ETMW;
    }

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  *pBytesWritten = byteCount;
  return (status);
}/*Mv300BrdWrite*/

/*----------------------- Device functions -----------------------------------*/
/*-----------------------------------------------------------------------------
 * Mv300BrdDevCommand (brdId, pConf, pBuf, bufLen, pBytesRead) : status
 *
 * Purpose:
 *  Send multiline commands from buffer using device configuration
 * Returns:
 *  status		: STATUS	- OK if no problems
 * Arguments:	     
 *  brdId		: MV300_BRD_ID	- pointer to board control structure
 *  pConf		: PMV300_CONFIG - pointer to device control structure
 *  pBuf		: BYTE*		- pointer to command array
 *  bufLen      	: UINT		- number of bytes in command array
 *  pBytesWritten	: UINT*		- pointer to variable for number
 *					  of bytes written
 * Description:
 *  Send IEEE488 multiline commands using device configuration *pConf.
 *  If board is not CIC, return error. pConf->priAddr is not used.
 * Note:
 *  Current implementation does not use DMA transfers. Most command sequences
 *  are relatively short and would not benefit from the greater setup time
 *  for DMA. However, by examining Mv300BrdDevWrite one could add DMA to
 *  this function.
 */
STATUS Mv300BrdDevCommand (brdId, pConf, pBuf, bufLen, pBytesWritten)
     MV300_BRD_ID  brdId;
     PMV300_CONFIG pConf;
     BYTE*	   pBuf;
     UINT	   bufLen;
     UINT*	   pBytesWritten;
{
  FAST MV300_PORT*  pPort;
  STATUS	    status = OK;
  INT	    	    index  = 0;
  
  /* test for valid board id */
  if (brdId == NULL)
    {
      *pBytesWritten = 0;
      return (ERROR_EBID);
    }

  /* fast pointer to port */
  pPort = (MV300_PORT*) brdId->port;

  /* grab control and start watchdog if timeout non-zero */
  semTake (brdId->brdSemId);
  brdId->state &= ~BRD_TIMEOUT;
  if (pConf->timeout != 0)
    wdStart (brdId->wDogId, pConf->timeout, brdTimeout, (INT) brdId);

  /* initialize interrupt masks and clear output buffer */
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;
  pPort->tms.w.aux_cmd   = CMD_NBAF;

  /* test for controller-in-charge */
  if (!(brdId->state & BRD_CIC))
    {
      *pBytesWritten = 0;
      status = ERROR_ECIC;
    }
  else /* send multiline commands */
    {
      /* take control */
      pPort->tms.w.int0_mask = IM0_BO;
      if (!(pPort->tms.r.addr_stat & ASR_ATN))
	{
	  pPort->tms.w.aux_cmd = CMD_TCA;
	  status = brdWait (brdId);
	}
      /* send commands */
      for (index = 0; (status == OK) && (index < bufLen); index++)
	{
	  pPort->tms.w.data_out  = pBuf[index];
	  status = brdWait (brdId);
	}
      *pBytesWritten = index;
      /* release the bus */
      pPort->tms.w.int0_mask = 0;
      pPort->tms.w.aux_cmd = CMD_GTS;
    }

  /* clear watchdog and test for timeout */
  wdCancel (brdId->wDogId);
  /* semaphore should only be set if timeout  before reaching this point */
  if (semClear (brdId->intSemId) == OK)	/* test and clear */
    {
      status = ERROR_ETMW;
    }
  
 /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (status);
}/*Mv300BrdDevCommand*/

/*-----------------------------------------------------------------------------
 * Mv300BrdDevRead (brdId, pConf, pBuf, bufLen, pBytesRead) : status
 *
 * Purpose:
 *  Read device data from GPIB device specified in pConf
 * Returns:
 *  status		: STATUS	 - OK if no problems
 * Arguments:			
 *  brdId		: MV300_BRD_ID	 - pointer to board control structure
 *  pConf		: PMV300_CONFIG - pointer to device control structure
 *  pBuf		: BYTE*		 - pointer to data byte array
 *  bufLen		: UINT		 - number of bytes in data array
 *  pBytesRead		: UINT*		 - pointer to variable for number
 *					   of bytes read.
 * Description:
 *  Address device to talk, board to listen, and transfer data into buffer.
 *  Return device to untalk and board to CACS.
 *  It is an error if board is not CIC.
 */
STATUS Mv300BrdDevRead (brdId, pConf, pBuf, bufLen, pBytesRead)
     MV300_BRD_ID         brdId;
     PMV300_CONFIG	  pConf;
     BYTE*		  pBuf;
     UINT		  bufLen;
     UINT*		  pBytesRead;
{
  FAST MV300_PORT*  pPort = NULL;
  STATUS	    status = OK;
  INT	    	    index = 0;
  INT		    limit = 0;
  UINT	    	    byteCount = 0;
  BYTE *	    busAdrs = 0;
  BYTE		    buf[3];

  /* test for valid board id */
  if (brdId == NULL)
    {
      *pBytesRead = 0;
      return (ERROR_EBID);
    }
  
  /* grab control and start watchdog */
  semTake (brdId->brdSemId);
  brdId->state &= ~BRD_TIMEOUT;
  if (pConf->timeout != 0)
    wdStart (brdId->wDogId, pConf->timeout, brdTimeout, (INT) brdId);
  
  /* initialize fast pointers */
  pPort = (MV300_PORT*) brdId->port;
  
  /* disable tms interrupts */
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;
  
  /* set up for listening */
  /* disable all holdoff except data RFD holdoff */
  pPort->tms.w.aux_cmd = CMD_HDFA_CLR;
  pPort->tms.w.aux_cmd = CMD_HDFE_CLR;
  pPort->tms.w.aux_cmd = CMD_DACR;

  /* error if not CIC */
  if (!(brdId->state & BRD_CIC))
    {
      status = ERROR_ECIC;
      *pBytesRead = 0;
    }
  else /* CIC */
    {
      /* take control */
      pPort->tms.w.int0_mask = IM0_BO;
      if (!(pPort->tms.r.addr_stat & ASR_ATN))
	{
	  pPort->tms.w.aux_cmd = CMD_TCA;
	  status = brdWait (brdId);
	}
      if (pConf->secAddr != 0) /* secondary addressing? */
	{
	  buf[0] = MV300_BRD_K_GPIB_UNL;
	  buf[1] = MV300_BRD_K_GPIB_TAG | pConf->priAddr;
	  buf[2] = MV300_BRD_K_GPIB_SCG | pConf->secAddr;
	  limit = 3;
	}
      else
	{
	  buf[0] = MV300_BRD_K_GPIB_UNL;
	  buf[1] = MV300_BRD_K_GPIB_TAG | pConf->priAddr;
	  limit = 2;
	}
      /* send address commands */
      for (index = 0; (status == OK) && (index < limit); index++)
	{
	  pPort->tms.w.data_out  = buf[index];
	  status = brdWait (brdId);
	}
      /* begin listening */
      pPort->tms.w.int0_mask = 0;
      pPort->tms.w.aux_cmd = CMD_LON_SET;
      pPort->tms.w.aux_cmd = CMD_GTS;
    }

  /* if status is OK do either dma or programmed read */
  if (status == OK)
    {
      /* read data either by dma or programmed io */
      if (pConf->flags & MV300_CONFIG_V_DMA)
	{ /* BEGIN DMA READ */
	  /* get vme bus address */
	  if (sysLocalToBusAdrs (VME_AM_STD_SUP_DATA, pBuf, &busAdrs) != OK)
	    {
	      byteCount = 0;
	      status = ERROR_EARG;
	    }
	  else /* valid vme bus address */
	    {
	      pPort->mar0 = MAR0 (busAdrs);
	      pPort->mar1 = MAR1 (busAdrs);
	      pPort->mar2 = MAR2 (busAdrs);
	      
	      /* load byte count register */
	      byteCount = bufLen;
	      pPort->bcr0 = BCNT0 (byteCount);
	      pPort->bcr1 = BCNT1 (byteCount);
	      
	      pPort->eoscr = pConf->reosChar;
	      
	      /* start dma read */
	      pPort->tms.w.int0_mask = IM0_END;
	      pPort->ucr = (pConf->flags & MV300_CONFIG_V_REOS) ?
		UCR_DMAW | UCR_EOS : UCR_DMAW;
	      
	      /* wait for interrupt */
	      status = brdWait (brdId);
	      
	      /* get number of bytes actually read */
	      byteCount = ((BYTE*) ((pPort->mar0 << 0) +
				    (pPort->mar1 << 8) +
				    (pPort->mar2 << 16))) - busAdrs - 1;

	      /* get last byte in input register */
	      pBuf[byteCount++] = pPort->tms.r.data_in;
	    }
	} /* END DMA READ */
      else
	{ /* BEGIN PROGRAMMED READ */
	  pPort->tms.w.int0_mask = IM0_BI;
	  for (byteCount = 0; byteCount < bufLen; byteCount++)
	    {
	      /* get next byte */
	      if ((status = brdWait (brdId)) != OK)
		break;
	      pBuf[byteCount] = pPort->tms.r.data_in;
	      /* EOS? */
	      if ((pConf->flags & MV300_CONFIG_V_REOS) &&
		  (pBuf[byteCount] == pConf->reosChar))
		{
		  byteCount++;
		  break;
		}
	      /* EOI? */
	      if (brdId->status.int0_stat & IM0_END)
		{
		  byteCount++;
		  break;
		}
	    }
	  pPort->tms.w.int0_mask = 0;
	} /* END PROGRAMMED READ */

      /* clean up */
      pPort->tms.w.int0_mask = IM0_BO;
      pPort->tms.w.aux_cmd = CMD_TCA;
      if (status == OK)
	status = brdWait (brdId);
      pPort->tms.w.data_out = MV300_BRD_K_GPIB_UNT;
      if (status == OK)
	status = brdWait (brdId);
      pPort->tms.w.aux_cmd = CMD_LON_CLR;
      pPort->tms.w.int0_mask = 0;
      buf[0] = pPort->tms.r.data_in; /* flush out input buffer */
      pPort->tms.w.aux_cmd = CMD_GTS;
    }	  /* END READ */

  /* clear watchdog and test for timeout */
  /* intSemId should only be set if timeout  before reaching this point */
  wdCancel (brdId->wDogId);
  if (semClear (brdId->intSemId) == OK)	/* test and clear */
    {
      status = ERROR_ETMW;
    }
 
 /* restore control of board and return */
  semGive (brdId->brdSemId);
  *pBytesRead = byteCount;
  return (status);
}/*Mv300BrdDevRead*/

/*-----------------------------------------------------------------------------
 * Mv300BrdDevSPoll (brdId, pConf, pResult) : status
 *
 * Purpose:
 *  Serial poll a device
 * Returns:
 *  status	: STATUS	- OK if no problems
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure.
 *  pConf	: PMV300_CONFIG	- pointer to device configuration structure.
 *  pResult	: BYTE*		- pointer to result.
 * Description:
 *  Serial poll device specified in pConf. Return result in pResult.
 * NOTE:
 */
STATUS Mv300BrdDevSPoll (brdId, pConf, pResult)
     MV300_BRD_ID   brdId;
     PMV300_CONFIG  pConf;
     BYTE*          pResult;
{
  FAST MV300_PORT*  pPort = NULL;
  STATUS	    status = OK;
  INT	    	    index = 0;
  INT		    limit = 0;
  BYTE		    buf[5];

  /* test for valid board id */
  if (brdId == NULL)
    {
      return (ERROR_EBID);
    }
  
  /* grab control and start watchdog */
  semTake (brdId->brdSemId);
  brdId->state &= ~BRD_TIMEOUT;
  if (pConf->timeout != 0)
    wdStart (brdId->wDogId, pConf->timeout, brdTimeout, (INT) brdId);
  
  /* initialize fast pointers */
  pPort = (MV300_PORT*) brdId->port;
  
  /* initialize tms interrupt mask */
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;
  
  /* error if not CIC */
  if (!(brdId->state & BRD_CIC))
    {
      status = ERROR_ECIC;
      *pResult = 0;
    }
  else /* CIC */
    {
      /* take control */
      pPort->tms.w.int0_mask = IM0_BO;
      if (!(pPort->tms.r.addr_stat & ASR_ATN))
	{
	  pPort->tms.w.aux_cmd = CMD_TCA;
	  status = brdWait (brdId);
	}
      if (pConf->secAddr != 0) /* secondary addressing? */
	{
	  buf[0] = MV300_BRD_K_GPIB_UNL;
	  buf[1] = MV300_BRD_K_GPIB_SPE;
	  buf[2] = MV300_BRD_K_GPIB_TAG | pConf->priAddr;
	  buf[3] = MV300_BRD_K_GPIB_SCG | pConf->secAddr;
	  limit = 4;
	}

      else
	{
	  buf[0] = MV300_BRD_K_GPIB_UNL;
	  buf[1] = MV300_BRD_K_GPIB_SPE;
	  buf[2] = MV300_BRD_K_GPIB_TAG | pConf->priAddr;
	  limit = 3;
	}
      /* send address commands */
      for (index = 0; (status == OK) && (index < limit); index++)
	{
	  pPort->tms.w.data_out  = buf[index];
	  status = brdWait (brdId);
	}
      /* listen and go to standby */
      pPort->tms.w.aux_cmd = CMD_LON_SET;
      pPort->tms.w.int0_mask = IM0_BI;
      pPort->tms.w.aux_cmd = CMD_GTS;
      /* read status byte */
      if (status == OK)
	status = brdWait (brdId);
      /* don't read byte until have control */
      pPort->tms.w.int0_mask = IM0_BO;
      pPort->tms.w.aux_cmd = CMD_TCS;
      if (status == OK)
	status = brdWait (brdId);
      *pResult = pPort->tms.r.data_in;
      /* clean up */
      /* note: we attempt to perform all of these commands */
      /* even if previous status is not OK */
      pPort->tms.w.data_out = MV300_BRD_K_GPIB_SPD;
      if (status == OK)
	status = brdWait (brdId);
      pPort->tms.w.data_out = MV300_BRD_K_GPIB_UNT;
      if (status == OK)
	status = brdWait (brdId);
      pPort->tms.w.int0_mask = 0;
      pPort->tms.w.aux_cmd = CMD_LON_CLR;
      pPort->tms.w.aux_cmd = CMD_GTS;
    }

  /* clear watchdog and test for timeout */
  /* intSemId should only be set if timeout  before reaching this point */
  wdCancel (brdId->wDogId);
  if (semClear (brdId->intSemId) == OK)	/* test and clear */
    {
      status = ERROR_ETMW;
    }
 
 /* restore control of board and return */
  semGive (brdId->brdSemId);
  return (status);
} /*Mv300BrdDevSPoll`*/

/*-----------------------------------------------------------------------------
 * Mv300BrdDevWrite (brdId, pConf, pBuf, bufLen, pBytesWritten) : status
 *
 * Purpose:
 *  Write to device specified in pConf
 * Returns:
 *  status		: STATUS	- OK if no problems
 * Arguments:			
 *  brdId		: MV300_BRD_ID 	 - pointer to board control structure
 *  pConf		: PMV300_CONFIG - pointer to device control structure
 *  pBuf		: BYTE*		 - pointer to data byte array
 *  bufLen		: UINT		 - number of bytes in data array
 *  pBytesWritten	: UINT*		 - pointer to variable for
 *					   number of bytes written.
 * Description:
 */
STATUS Mv300BrdDevWrite (brdId, pConf, pBuf, bufLen, pBytesWritten)
     MV300_BRD_ID       brdId;
     PMV300_CONFIG	pConf;
     BYTE*		pBuf;
     UINT		bufLen;
     UINT*		pBytesWritten;
{
  FAST MV300_PORT*  pPort = NULL;
  STATUS	    status = OK;
  INT	    	    index = 0;
  int		    limit = 0;
  UINT		    byteCount = 0;
  BYTE *	    busAdrs = 0;
  BYTE		    buf[4];
  
  /* test for valid board id */
  if (brdId == NULL)
    {
      *pBytesWritten = 0;
      return (ERROR_EBID);
    }
  /* else grab control and start watchdog */
  semTake (brdId->brdSemId);
  brdId->state &= ~BRD_TIMEOUT;
  if (pConf->timeout != 0)
    wdStart (brdId->wDogId, pConf->timeout, brdTimeout, (INT) brdId);
  
  /* initialize fast pointer */
  pPort = (MV300_PORT*) brdId->port;

  /* clear out data_out register */
  pPort->tms.w.aux_cmd = CMD_NBAF;
  
  /* mask all interrupts */
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;
  
  /* error if not CIC */
  if (!(brdId->state & BRD_CIC))
    {
      *pBytesWritten = 0;
      status = ERROR_ECIC;
    }
  else /* CIC */
    {
      /* take control */
      pPort->tms.w.int0_mask = IM0_BO;
      if (!(pPort->tms.r.addr_stat & ASR_ATN))
	{
	  pPort->tms.w.aux_cmd = CMD_TCA;
	  status = brdWait (brdId);
	}
      if (pConf->secAddr != 0) /* secondary addressing? */
	{
	  buf[0] = MV300_BRD_K_GPIB_UNL;
	  buf[1] = MV300_BRD_K_GPIB_LAG | pConf->priAddr;
	  buf[2] = MV300_BRD_K_GPIB_SCG | pConf->secAddr;
	  buf[3] = MV300_BRD_K_GPIB_TAG | brdId->config.priAddr;
	  limit = 4;
	}
      else
	{
	  buf[0] = MV300_BRD_K_GPIB_UNL;
	  buf[1] = MV300_BRD_K_GPIB_LAG | pConf->priAddr;
	  buf[2] = MV300_BRD_K_GPIB_TAG | brdId->config.priAddr;
	  limit = 3;
	}
      /* send address commands */
      for (index = 0; (status == OK) && (index < limit); index++)
	{
	  pPort->tms.w.data_out  = buf[index];
	  status = brdWait (brdId);
	}
      /* goto standby and wait for output port ready */
      pPort->tms.w.aux_cmd = CMD_TON_SET;
      pPort->tms.w.aux_cmd = CMD_GTS;
      /* wait for TADS/TACS state */
      if (status == OK)
	status = brdWait (brdId);
      pPort->tms.w.int0_mask = 0;
    }

  /* if status is OK do either dma or programmed write */
  if (status == OK)
    { /* BEGIN WRITE */
      /* find byte count */
      byteCount = (pConf->flags & MV300_CONFIG_V_WEOS) ? bufLen : bufLen - 1;
      if (pConf->flags & MV300_CONFIG_V_DMA)
	{ /* BEGIN DMA WRITE */
	  /* get vme bus address */
	  if (sysLocalToBusAdrs (VME_AM_STD_SUP_DATA, pBuf, &busAdrs) != OK)
	    {
	      *pBytesWritten = 0;
	      status = ERROR_EARG;
	    }
	  else
	    {
	      pPort->mar0 = MAR0 (busAdrs);
	      pPort->mar1 = MAR1 (busAdrs);
	      pPort->mar2 = MAR2 (busAdrs);
	      
	      /* load byte count register */
	      pPort->bcr0 = BCNT0 (byteCount);
	      pPort->bcr1 = BCNT1 (byteCount);
	      
	      /* start dma write */
	      pPort->ucr = UCR_DMAR;
	      
	      /* wait for interrupt */
	      status = brdWait (brdId);
	      
	      /* get number of bytes actually read */
	      byteCount = ((BYTE*) ((pPort->mar0 << 0) +
				    (pPort->mar1 << 8) +
				    (pPort->mar2 << 16))) - busAdrs;
	    }
	} /* END DMA WRITE */
      else /* Programmed IO */
	{ /* BEGIN PROGRAMMED WRITE */
	  /* send all but last byte */
	  pPort->tms.w.int0_mask = IM0_BO;
	  for (index = 0; (status == OK) && (index < byteCount); index++)
	    {
	      pPort->tms.w.data_out = pBuf[index];
	      status = brdWait (brdId);
	    }
	  byteCount = index;
	  pPort->tms.w.int0_mask = 0;
	} /* END PROGRAMMED WRITE */
      /* if OK so far, send last byte with EOI */
      if (status == OK)
	{
	  pPort->tms.w.int0_mask = IM0_BO;
	  pPort->tms.w.aux_cmd = CMD_FEOI;
	  if (pConf->flags & MV300_CONFIG_V_WEOS)
	    {
	      pPort->tms.w.data_out = pConf->weosChar;
	      status = brdWait (brdId);
	    }
	  else
	    {
	      pPort->tms.w.data_out = pBuf[bufLen - 1];
	      if ((status = brdWait (brdId)) == OK)
		byteCount++;
	    }
	  pPort->tms.w.int0_mask = 0;
	}
    } /* END WRITE */
  
  /* clean up */
  if (brdId->state & BRD_CIC)
    {
      pPort->tms.w.int0_mask = IM0_BO;
      pPort->tms.w.aux_cmd = CMD_TCA;
      if (status == OK)
	status = brdWait (brdId);
      pPort->tms.w.data_out = MV300_BRD_K_GPIB_UNL;
      if (status == OK)
	status = brdWait (brdId);
      pPort->tms.w.int0_mask = 0;
      pPort->tms.w.aux_cmd = CMD_TON_CLR;
      pPort->tms.w.aux_cmd = CMD_GTS;
    }

  /* clear watchdog and test for timeout */
  /* intSemId should only be set if timeout  before reaching this point */
  wdCancel (brdId->wDogId);
  if (semClear (brdId->intSemId) == OK)	/* test and clear */
    {
      status = ERROR_ETMW;
    }

  /* restore control of board and return */
  semGive (brdId->brdSemId);
  *pBytesWritten = byteCount;
  return (status);
}/*Mv300BrdDevWrite*/

/*========================== Local Functions =================================*/
/*-----------------------------------------------------------------------------
 * brdInt (brdId) : void
 *	
 * Purpose:
 *   Main exported interrupt handler.
 * Returns:
 * Arguments:
 *   brdId	: MV300_BRD_ID	- pointer to MV300_BRD control structure
 * Description:
 *   This routine handles interrupts from the MV300 board.
 */
LOCAL VOID brdInt (brdId)
     MV300_BRD_ID brdId;
{
  FAST MV300_PORT* pPort = (MV300_PORT*) brdId->port;

  /* get status */
  brdId->status.brd_stat  = pPort->csreg & CSR_MASK;
  brdId->status.int0_stat = pPort->tms.r.int0_stat & IM0_MASK;
  brdId->status.int1_stat = pPort->tms.r.int1_stat;
  brdId->status.addr_stat = pPort->tms.r.addr_stat;

  /* clear any bus errors and board done flags */
  pPort->ucr = UCR_CLEAR;
  semGive (brdId->intSemId);

}/*brdInt*/

/*-----------------------------------------------------------------------------
 * brdOffline (brdId) : void
 *
 * Purpose:
 *  Place board offline
 * Returns:
 * Arguments:
 *  brdId	: MV300_BRD_ID	- pointer to board control structure
 * Description:
 *  Disable interrupts and reset board and TMS9914A states.
 */
LOCAL VOID brdOffline (brdId)
     MV300_BRD_ID       brdId;
{
  FAST MV300_PORT*  pPort = (MV300_PORT*) brdId->port;

  /* disable interrupts */
  pPort->ucr = UCR_CLEAR;
  pPort->cr1 = BIM_DIS;
  pPort->cr2 = BIM_DIS;
  pPort->cr3 = BIM_DIS;

  /* reset TMS9914A */
  pPort->tms.w.aux_cmd   = CMD_SWRST_SET;
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;
  pPort->tms.w.aux_cmd   = CMD_SWRST_CLR;

}/*brdOffline*/

/*-----------------------------------------------------------------------------
 * brdOnline (brdId) : VOID
 *
 * Purpose:
 *  Place the board online.
 * Returns:
 * Arguments:
 *  brdId	: MV300_BRD_ID		- pointer to board control structure
 * Description:
 *  Initialize board interface registers and enable interrupts.
 */
LOCAL VOID brdOnline (brdId)
     MV300_BRD_ID 	brdId;
{
  FAST MV300_PORT*  pPort = (MV300_PORT*) brdId->port;
  
  /* initialize board registers */
  pPort->amr	 = VME_AM_STD_SUP_DATA;		/* setup address modifier */
  pPort->vr0	 = (BYTE) brdId->intNumber;	/* setup interrupts */
  pPort->vr1	 = (BYTE) brdId->intNumber;
  pPort->vr2	 = (BYTE) brdId->intNumber;
  pPort->vr3	 = 0;
  pPort->cr0	 = brdId->intLevel | BIM_IRE;	/* enable interrupts */
  pPort->cr1	 = brdId->intLevel | BIM_IRE;
  pPort->cr2	 = brdId->intLevel | BIM_IRE;
  pPort->cr3	 = 0;
  pPort->vbrcr	 = REL_CNT;			/* release count */

  /* initialize the TMS9914A registers */
  busInit (brdId);				/* initialize bus */

}/*brdOnline*/

/*-----------------------------------------------------------------------------
 * brdTimeout (brdId) : VOID
 *
 * Purpose:
 *  Handle watchdog timout.
 * Returns:
 * Arguments:
 *  brdId	: MV300_BRD*	- pointer to board control structure
 * Description:
 */
LOCAL VOID brdTimeout (brdId)
     MV300_BRD*	brdId;
{
  FAST MV300_PORT*	pPort = (MV300_PORT*) brdId->port;
  
  /* set timeout error flag, disable interrupts and give semaphore */
  brdId->state |= BRD_TIMEOUT;
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;
  pPort->ucr = UCR_CLEAR;
  semGive (brdId->intSemId);

/*BEGDEBUG*/
  logMsg ("Mv300Brd: Timeout!!!!!!!!!!!!!!!!\n");
/*ENDDEBUG*/
}/*brdTimeout*/

/*-----------------------------------------------------------------------------
 * brdWait (brdId) : status
 *
 * Purpose:
 *  Wait for return from interrupt and process before returning status
 * Returns:
 *  status	: STATUS	- OK if no errors
 * Arguments:
 *  brdId	: MV300_BRD_ID		- pointer to board control structure
 * Description:
 *  Wait at intSemId for interrupt return. Process interrupt according
 *  to return state.
 */

LOCAL STATUS brdWait (brdId)
     MV300_BRD_ID	brdId;
{
  FAST MV300_PORT* pPort = (MV300_PORT*) brdId->port;
  STATUS	status = OK;
  BYTE		test1, test2;

  /* wait for interrupt return */
  semTake (brdId->intSemId);

  /* process interrupt */
  /* timeout? */
  if (brdId->state & BRD_TIMEOUT)
    {
      busInit (brdId);			/* initialize bus */
      status = ERROR_ETMW;
    }
  /* bus error? */
  else if (brdId->status.brd_stat & 0x20) /* BERR */
    {
      status = ERROR_EBUS;
    }
  /* else everything was ok */
  return (status);
}/*brdWait*/

/*-----------------------------------------------------------------------------
 * busInit (brdId) : void
 *
 * Purpose:
 *   Initialize bus interface hardware.
 * Returns:
 * Arguments:
 *   brdId	: MV300_BRD*	- pointer to controller structure
 * Description:
 *   Initialize TMS9914A bus interface.
 */
LOCAL VOID busInit (brdId)
    MV300_BRD_ID brdId;
{
  FAST MV300_PORT *pPort = (MV300_PORT*) brdId->port;
  
  pPort->tms.w.aux_cmd	 = CMD_SWRST_SET;
  pPort->tms.w.addr_reg	 = pPort->tms.r.addr_sw1 & 0x1f;
  pPort->tms.w.int0_mask = 0;
  pPort->tms.w.int1_mask = 0;
  pPort->tms.w.ser_poll	 = 0;
  pPort->tms.w.par_poll	 = 0;
  pPort->tms.w.aux_cmd	 = CMD_RSV2_CLR;
  pPort->tms.w.aux_cmd	 = CMD_FGET_CLR;
  /* If system controller, clear bus and set remote enable */
  if (SYS_CTRL (pPort))
    {
      brdId->state |= BRD_SC;			/* set system controller bit */
      pPort->tms.w.aux_cmd = CMD_SIC_SET;	/* send interface clear */
      pPort->tms.w.aux_cmd = CMD_SWRST_CLR;
      taskDelay (1);				/* wait at least 100us */
      pPort->tms.w.aux_cmd = CMD_SIC_CLR;	/* reset interface clear */
      /* NOTE: the proper method here would be to do a brdWait on IM0_BO */
      /* However, this function is called by brdWait itself in the event */
      /* of a timeout. We do not want to hang up the system in this case */
      /* So, we fake it by waiting one clock tick and proceeding.        */
      taskDelay (1);				/* wait at least 100us */
      brdId->state |= BRD_CIC;			/* Set cic status bit */
      pPort->tms.w.aux_cmd = CMD_SRE_SET;	/* set remote enable */
      pPort->tms.w.aux_cmd = CMD_GTS;
    }
  /* else just clear software reset and initialize state bits */
  else
    {
      pPort->tms.w.aux_cmd = CMD_SWRST_CLR;
      brdId->state &= ~(BRD_SC | BRD_CIC);
    }
}/* busInit */
/*=============================== End ========================================*/
