/* $Id: Mv300Brd.h,v 1.8 91/02/06 11:38:46 cwcork Exp Locker: cwcork $
 *
 * $File: Mv300Brd.h $
 *
 * $Revision: 1.8 $
 * $State: Exp $
 * $Author: cwcork $
 *
 * (C) Copyright 1990 Carl Cork. All rights reserved.
 * Permission to copy for any purpose is hereby granted so long
 * as this copyright notice remains intact.
 *
 * $Log:	Mv300Brd.h,v $
 * Revision 1.8  91/02/06  11:38:46  cwcork
 * Changes to use ../include files.
 * 
 * Revision 1.7  91/01/26  09:00:29  cwcork
 * Replace $ with _ in all defined constants.
 * 
 * Revision 1.6  90/08/29  10:13:48  cwcork
 * Move hidden part of MV300_BRD struct into this module.
 * Redefine MV300_BRD_ID as selector instead of pointer,
 *   but use it as a pointer. (Will not work with ANSI-C prototypes).
 * This is done to make RPC networking transparent.
 * 
 * Revision 1.5  90/07/27  17:58:05  cwcork
 * add state element to struct MV300_BRD.
 * update function prototypes.
 * 
 * Revision 1.4  90/07/10  16:21:14  cwcork
 * Split into two parts: Mv300.h and Mv300Brd.h.
 * Add prototypes for new functions.
 * 
 * Revision 1.3  90/06/21  16:20:58  cwcork
 * Add error status for Mv300BrdCreate.
 * Revise prototypes:
 *   Mv300BrdCreate now declared to be of type STATUS.
 *   Mv300BrdRd -> Mv300BrdRead.
 *   Mv300BrdWrt -> Mv300BrdWrite.
 *   Add new function Mv300BrdAddr.
 * 
 * Revision 1.2  90/06/15  17:08:46  cwcork
 * Add interrupt status storage to MV300BRD structure
 * Add call to Mv300BrdStatus ().
 * 
 * Revision 1.1  90/06/04  13:12:05  cwcork
 * Initial revision
 * 
 *
 * PURPOSE:
 *  Descriptor File, Board Control Module
 *  for Motorola MVME300 IEEE-488 (GPIB) bus controller.
 *
 * COMMENTS:
 * KNOWN BUGS:
 */
#ifndef MV300BRD_H
#define MV300BRD_H 1

/*======================== External Declarations =============================*/
#include "Mv300.h"
#include "wdLib.h"
#include "semLib.h"

/* Multiline GPIB commands */

#define MV300_BRD_K_GPIB_DCL		0x14
#define MV300_BRD_K_GPIB_GET		0x08
#define MV300_BRD_K_GPIB_GTL		0x01
#define MV300_BRD_K_GPIB_LAG		0x20
#define MV300_BRD_K_GPIB_LLO		0x11
#define MV300_BRD_K_GPIB_PPC		0x05
#define MV300_BRD_K_GPIB_PPD		0x70
#define MV300_BRD_K_GPIB_PPE		0x60
#define MV300_BRD_K_GPIB_PPU		0x15
#define MV300_BRD_K_GPIB_SCG		0x60
#define MV300_BRD_K_GPIB_SDC		0x04
#define MV300_BRD_K_GPIB_SPD		0x19
#define MV300_BRD_K_GPIB_SPE		0x18
#define MV300_BRD_K_GPIB_TCT		0x09
#define MV300_BRD_K_GPIB_TAG		0x40
#define MV300_BRD_K_GPIB_UNL		0x3f
#define MV300_BRD_K_GPIB_UNT		0x5f

/* Board Control ID - equivalenced to PMV300_BRD inside Mv300Brd.c */

#ifndef MV300BRD_C
typedef unsigned int MV300_BRD_ID;
#endif  MV300BRD_C

/*======================== Exported Function Prototypes ======================*/

/*------------------------- Create and Forget --------------------------------*/
IMPORT STATUS Mv300BrdCreate ();
IMPORT STATUS Mv300BrdForget ();
/*------------------------- Board Control Operations -------------------------*/
IMPORT STATUS Mv300BrdAtn ();		/* Raise/Lower ATN */
IMPORT STATUS Mv300BrdCommand ();	/* Send multiline commands */
IMPORT STATUS Mv300BrdConfRd ();	/* Read board configuration */
IMPORT STATUS Mv300BrdConfWrt ();	/* Write board configuration */
IMPORT STATUS Mv300BrdLocal ();		/* Set/Clear rtl request */
IMPORT STATUS Mv300BrdOffline ();	/* Place board offline */
IMPORT STATUS Mv300BrdOnline ();	/* Place board online */
IMPORT STATUS Mv300BrdPPoll ();		/* Perform parallel poll on bus */
IMPORT STATUS Mv300BrdPPollByte ();	/* Write to parallel poll register */
IMPORT STATUS Mv300BrdRead ();		/* Read data from data_in port */
IMPORT STATUS Mv300BrdRemote ();	/* Set/Clear remote enable */
IMPORT STATUS Mv300BrdRequest ();	/* Set/Clear request service */
IMPORT STATUS Mv300BrdSendIfc ();	/* Send IFC command */
IMPORT STATUS Mv300BrdSPollByte ();	/* Write to serial poll register */
IMPORT STATUS Mv300BrdStatus ();	/* Get current board status */
IMPORT STATUS Mv300BrdWrite ();		/* Write to data_out register */
/*------------------------ Device Read and Write -----------------------------*/
IMPORT STATUS Mv300BrdDevCommand ();	/* Send multiline device commands */
IMPORT STATUS Mv300BrdDevRead ();	/* Read device data */
IMPORT STATUS Mv300BrdDevSPoll ();	/* Serial poll a device */
IMPORT STATUS Mv300BrdDevWrite ();	/* Write device data */
/*========================= Exported Variables ===============================*/
/*================================= END ======================================*/
#endif Mv300Brd_H
