/* $Id: Mv300BrdTest.c,v 1.1 91/02/06 11:39:17 cwcork Exp Locker: cwcork $
 *
 * $File: Mv300BrdTest.c$
 *
 * $Revision: 1.1 $
 *
 * $State: Exp $
 *
 * $Author: cwcork $
 *
 * (C) Copyright 1990 Carl Cork. All rights reserved.
 * Permission to copy for any purpose is hereby granted so long
 * as this copyright notice remains intact.
 *
 * $Log:	Mv300BrdTest.c,v $
 * Revision 1.1  91/02/06  11:39:17  cwcork
 * Initial revision
 * 
 * PURPOSE:
 *   Source file,
 *   for Mv300Brd module test routine
 *
 * FUNCTIONS:
 * COMMENTS:
 * KNOWN BUGS:
 */
#include "vxWorks.h"
#include "fioLib.h"
#include "Mv300Brd.h"

Mv300BrdTest ()
{
  MV300_BRD_ID	brdId;
  MV300_CONFIG  devConfig;
  MV300_STATUS  brdStatus;
  STATUS	status;
  STRING	host = "esg2";
  BYTE 		bufOut[10];
  BYTE		bufIn[16];
  UINT		bufOutLen = sizeof (bufOut);
  UINT		bufInLen = sizeof (bufIn) - 1;
  UINT		bytesTransferred = 0;

  /* initialize buffers */
  sprintf (bufOut, "ID?");
  bufOutLen = strlen (bufOut);
  sprintf (bufIn, "ERROR");
  
  printf ("\n------Mv300Brd link test------\n\n");

  /* set up ios structure */
  devConfig.timeout = 300;	/* timeout after 5 seconds (300 ticks) */
  devConfig.flags = (MV300_CONFIG_V_REOS |
		     MV300_CONFIG_V_WEOS |
                     MV300_CONFIG_V_DMA  |
		     MV300_CONFIG_V_EOT);
  devConfig.priAddr = 22;	        /* primary address */
  devConfig.secAddr = 0;	        /* no secondary addressing */
  devConfig.reosChar = '\r';
  devConfig.weosChar = '\r';

  printf ("Mv300BrdTest: devConfig timeout = %d, flags = 0x%x\n",
	  devConfig.timeout, devConfig.flags);
  printf ("Mv300BrdTest: devConfig priAddr = %d, secAddr = %d\n",
	  devConfig.priAddr, devConfig.secAddr);

  if ((status = Mv300BrdCreate (&brdId,
				host, 0xffff0100, 200, 3, &devConfig)) != OK)
    {
      printf ("Mv300BrdTest: Mv300BrdCreate Failure, status = %d\n", status);
      return;
    }
  else
    {
      printf ("Mv300BrdTest: Mv300BrdCreate OK, brdId = 0x%x\n", brdId);
    }

  /* setup remote enable */
  if ((status = Mv300BrdRemote (brdId, TRUE)) == OK)
    {
      printf ("Mv300BrdTest: Mv300BrdRemote: REN OK\n");
    }
  else
    {
      printf ("Mv300BrdTest: Mv300BrdRemote: REN FAIL, STATUS = %d\n", status);
      return;
    }
  /* print return status */
  Mv300BrdStatus (brdId, &brdStatus);
  printf ("Mv300BrdTest: Mv300BrdStatus return status: 0x%lx\n", brdStatus);

  printf ("Mv300BrdTest: Begin Mv300BrdDevWrite, bufOut = %s, bufOutLen = %d\n",
	  bufOut, bufOutLen);
  /* send query to 3458A */
  if ((status = Mv300BrdDevWrite (brdId, &devConfig, bufOut, bufOutLen,
			       &bytesTransferred)) == OK)
    {
      printf ("Mv300BrdTest: Mv300BrdDevWrite OK\n");
      printf ("Mv300BrdTest: Mv300BrdDevWrite bytesTransferred = %d\n",
	      bytesTransferred);
    }
  else
    {
      printf ("Mv300BrdTest: Mv300BrdDevWrite FAIL, STATUS = %d\n", status);
      printf ("Mv300BrdTest: Mv300BrdDevWrite bytesTransferred = %d\n",
	      bytesTransferred);
      return;
    }
  /* print return status */
  Mv300BrdStatus (brdId, &brdStatus);
  printf ("Mv300BrdTest: Mv300BrdStatus return status: 0x%lx\n", brdStatus);

  /* read back response */
  printf ("Mv300BrdTest: Begin Mv300BrdDevRead\n");
  if ((status = Mv300BrdDevRead (brdId, &devConfig, bufIn, bufInLen,
			      &bytesTransferred)) == OK)
    {
      bufIn[bytesTransferred] = '\0';
      printf ("Mv300BrdTest: Mv300BrdDevRead STATUS =   OK (0)\n");
      printf ("Mv300BrdTest: Mv300BrdDevRead bytesTransferred = %d\n",
	      bytesTransferred);
      printf ("Mv300BrdTest: Mv300BrdDevRead bufIn = %s\n", bufIn);
    }
  else
    {
      bufIn[bytesTransferred] = '\0';
      printf ("Mv300BrdTest: Mv300BrdDevRead STATUS = FAIL (%d)\n", status);
      printf ("Mv300BrdTest: Mv300BrdDevRead bytesTransferred = %d\n",
	      bytesTransferred);
      printf ("Mv300BrdTest: Mv300BrdDevRead bufIn = %s\n", bufIn);
      return;
    }
  /* print return status */
  Mv300BrdStatus (brdId, &brdStatus);
  printf ("Mv300BrdTest: Mv300BrdStatus return status: 0x%lx\n", brdStatus);

  /* clear remote enable */
  if ((status = Mv300BrdRemote (brdId, FALSE)) == OK)
    {
      printf ("Mv300BrdTest: Mv300BrdRemote REN CLEAR OK\n");
    }
  else
    {
      printf ("Mv300BrdTest: Mv300BrdRemote REN CLEAR FAIL, STATUS = %d\n", status);
      return;
    }
  /* print return status */
  Mv300BrdStatus (brdId, &brdStatus);
  printf ("Mv300BrdTest: Mv300BrdStatus return status: 0x%lx\n", brdStatus);

  /* Free up resources */
  printf ("Mv300BrdTest: Mv300BrdForget\n");
  if ((status = Mv300BrdForget (brdId)) == OK)
    {
      printf ("Mv300BrdTest: Mv300BrdForget STATUS =   OK (0)\n");
    }
  else
    {

      printf ("Mv300BrdTest: Mv300BrdForget STATUS = FAIL (%d)\n", status);
    }

  return;
}
