/* $Id: Mv300DevTest.c,v 1.1 91/02/06 11:40:21 cwcork Exp Locker: cwcork $
 *
 * $File: Mv300DevTest.c $
 *
 * $Revision: 1.1 $
 * $State: Exp $
 * $Author: cwcork $
 *
 * $Log:	Mv300DevTest.c,v $
 * Revision 1.1  91/02/06  11:40:21  cwcork
 * Initial revision
 * 
 * PURPOSE:
 *   test of Mv300Dev module
 *
 * COMMENTS:
 *   This test currently checks only a simple communication protocol with
 *   and HP3458A DVM. Query: ID?, Response: HP3458A.
 * KNOWN BUGS:
 */
#include "vxWorks.h"
#include "fioLib.h"
#include "Mv300Dev.h"
#include "Mv300Brd.h"

Mv300DevTest ()
{
  MV300_BRD_ID	brdId;
  MV300_DEV_ID  devId;
  MV300_CONFIG  devConfig;
  MV300_STATUS  brdStatus;
  STATUS	status;
  BYTE 		bufOut[10];
  BYTE		bufIn[16];
  UINT		bufOutLen = sizeof (bufOut);
  UINT		bufInLen = sizeof (bufIn) - 1;
  UINT		bytesTransferred = 0;

  /* initialize buffers */
  sprintf (bufOut, "ID?");
  bufOutLen = strlen (bufOut);
  sprintf (bufIn, "ERROR");
  
  printf ("\n------Mv300Dev test------\n\n");

  /* set up ios structure */
  devConfig.timeout = 0;	/* disable timeouts */
  devConfig.flags = (MV300_CONFIG_V_REOS |
		     MV300_CONFIG_V_WEOS |
                     MV300_CONFIG_V_DMA  |
		     MV300_CONFIG_V_EOT);
  devConfig.priAddr = 22;	        /* primary address */
  devConfig.secAddr = 0;	        /* no secondary addressing */
  devConfig.reosChar = '\r';
  devConfig.weosChar = '\r';

  printf ("Mv300DevTest: devConfig timeout = %d, flags = 0x%x\n",
	  devConfig.timeout, devConfig.flags);
  printf ("Mv300DevTest: devConfig priAddr = %d, secAddr = %d\n",
	  devConfig.priAddr, devConfig.secAddr);

  if ((status = Mv300BrdCreate (&brdId,
				"esg2", 0xffff0100, 200, 3, &devConfig)) != OK)
    {
      printf ("Mv300DevTest: Mv300BrdCreate Failure, status = %d\n", status);
      return;
    }
  else
    {
      printf ("Mv300DevTest: Mv300BrdCreate OK, brdId = 0x%x\n", brdId);
    }

  /* inialize device structure */
  if ((status = Mv300DevCreate (&devId, brdId, &devConfig)) != OK)
    {
      printf ("Mv300DevTest: Mv300DevCreate Failure, status = %d\n", status);
      return;
    }
  else
    {
      printf ("Mv300DevTest: Mv300DevCreate OK, brdId = 0x%x\n", brdId);
    }

  /* send query to 3458A */
  if ((status = Mv300DevWrite (devId, bufOut, bufOutLen,
			       &bytesTransferred)) == OK)
    {
      printf ("Mv300DevTest: Mv300DevWrite OK\n");
      printf ("Mv300DevTest: Mv300DevWrite bytesTransferred = %d\n",
	      bytesTransferred);
    }
  else
    {
      printf ("Mv300DevTest: Mv300DevWrite FAIL, STATUS = %d\n", status);
      printf ("Mv300DevTest: Mv300DevWrite bytesTransferred = %d\n",
	      bytesTransferred);
      return;
    }
  /* print return status */
  Mv300DevStatus (devId, &brdStatus);
  printf ("Mv300DevTest: Mv300DevStatus return status: 0x%lx\n", brdStatus);

  /* read back response */
  printf ("Mv300DevTest: Begin Mv300DevRead\n");
  if ((status = Mv300DevRead (devId, bufIn, bufInLen,
			      &bytesTransferred)) == OK)
    {
      bufIn[bytesTransferred] = '\0';
      printf ("Mv300DevTest: Mv300DevRead STATUS =   OK (0)\n");
      printf ("Mv300DevTest: Mv300DevRead bytesTransferred = %d\n",
	      bytesTransferred);
      printf ("Mv300DevTest: Mv300DevRead bufIn = %s\n", bufIn);
    }
  else
    {
      bufIn[bytesTransferred] = '\0';
      printf ("Mv300DevTest: Mv300DevRead STATUS = FAIL (%d)\n", status);
      printf ("Mv300DevTest: Mv300DevRead bytesTransferred = %d\n",
	      bytesTransferred);
      printf ("Mv300DevTest: Mv300DevRead bufIn = %s\n", bufIn);
      return;
    }
  /* print return status */
  Mv300DevStatus (devId, &brdStatus);
  printf ("Mv300DevTest: Mv300DevStatus return status: 0x%lx\n", brdStatus);

  /* go to local */
  if ((status = Mv300DevLocal (devId)) == OK)
    {
      printf ("Mv300DevTest: Mv300DevLocal GTL OK\n");
    }
  else
    {
      printf ("Mv300DevTest: Mv300DevLocal GTL FAIL, STATUS = %d\n", status);
      return;
    }
  /* print return status */
  Mv300DevStatus (devId, &brdStatus);
  printf ("Mv300DevTest: Mv300DevStatus return status: 0x%lx\n", brdStatus);

  /* Free up resources */
  printf ("Mv300DevTest: Mv300DevForget\n");
  if ((status = Mv300DevForget (devId)) == OK)
    {
      printf ("Mv300DevTest: Mv300DevForget STATUS =   OK (0)\n");
    }
  else
    {

      printf ("Mv300DevTest: Mv300DevForget STATUS = FAIL (%d)\n", status);
    }

  printf ("Mv300DevTest: Mv300BrdForget\n");
  if ((status = Mv300BrdForget (brdId)) == OK)
    {
      printf ("Mv300DevTest: Mv300BrdForget STATUS =   OK (0)\n");
    }
  else
    {

      printf ("Mv300DevTest: Mv300BrdForget STATUS = FAIL (%d)\n", status);
    }

  return;
}
