/* $Id: Mv300Drv.h,v 1.2 91/02/06 11:38:54 cwcork Exp Locker: cwcork $
 *
 * $Workfile: Mv300Drv.h $
 * $Author: cwcork $
 *
 * $Log:	Mv300Drv.h,v $
 * Revision 1.2  91/02/06  11:38:54  cwcork
 * Changes to use ../include files.
 * 
 * Initial Code
 *
 * PURPOSE:
 *  Device driver for Motorola MVME300 IEEE-488 (GPIB) bus controller module
 *
 * FUNCTIONS:
 *  exported-
 *  local-
 *
 * COMMENTS:
 * KNOWN BUGS:
 */
#ifndef MV300DRV_H
#define MV300DRV_H 1
/*------------------------ External Declarations -----------------------------*/
#include "semLib.h"
#include "rngLib.h"
#include "msgQLib.h"

typedef char*           ADDR, STRING;
typedef          char   CHAR, BOOL_1, INT_1;
typedef unsigned char   BYTE, UCHAR, UINT_1;
typedef          short  INT_2, BOOL_2;
typedef unsigned short  UINT_2;
typedef          int    INT, BOOL;
typedef unsigned int    UINT;
typedef          long   INT_4, BOOL_4;
typedef unsigned long   UINT_4;
typedef          float  REAL_4, REAL;
typedef          double REAL_8;

/* HIDDEN */

#define MV300_BUF_SIZE	64

typedef struct 		/* MV300_MOD - MV300 controller structure */
{
  ADDR         port;			/* MV300 i/o port base address */
  WDOG_ID      wDog;		        /* watchdog timer */
  SEM_ID       modSemId;	        /* module mutex id for exclusive access to MV300 */
  SEM_ID       intSemId;		/* interrupt semaphore id */
  struct
    {
      BYTE csreg;			/* module status register */
      BYTE int0_stat;			/* current int0 status register */
      BYTE int1_stat;			/* current int1 status register */
      BYTE addr_stat;			/* current gpib addressing status */
      BYTE bus_stat;			/* current gpib bus status */
      BYTE cmd_pass;			/* current contents of command pass-thru register */
    }status;	    
  MV300_DEV_ID devId;			/* current device structure */
  LIST         dependents;		/* list of dependents waiting on interrupt flags */
  CHAR         cbuf[MV300_BUF_SIZE];	/* character buffer */
} MV300_MOD;

typedef struct		/* MV300_DEV - MV300 device descriptor */
{
  DEV_HDR       tyDev    	/* tty device descriptor */
  MV300_MOD_ID  modId;		/* associated module control structure */
  BYTE		priAddr;
  BYTE		secAddr;
  UINT  	timeout;
  BYTE		eosChar;
  BYTE		state;
  struct
    {
      unsigned ueot:1;		/* enable EOI on write */
      unsigned ureos:1;		/* terminate read on EOS */
      unsigned uweos:1;		/* send EOS on write (with EOI) */
      unsigned udma:1;		/* do DMA I/O transfers */
      unsigned usc:1;		/* device is system controller */
      unsigned unad:1;		/* unaddress after device function */
      unsigned reserved:10;	/* pad out to 16 bits */
    } config;
} MV300_DEV;

/* END HIDDEN */

typedef MV300_DEV* MV300_DEV_ID;
typedef MV300_MOD* MV300_MOD_ID;

/*------------------------ Exported Function Prototypes ----------------------*/
/*------------------------ Exported Variables --------------------------------*/
/*--------------------------------- END --------------------------------------*/
#endif INCHBDRVH
